@@ -58,6 +58,8 @@ import org.apache.empire.jsf2.controls.TextInputControl;
 import org.apache.empire.jsf2.impl.FacesImplementation;
 import org.apache.empire.jsf2.impl.ResourceTextResolver;
 import org.apache.empire.jsf2.pages.Page;
+import org.apache.empire.jsf2.pages.PageDefinition;
+import org.apache.empire.jsf2.utils.BeanScope;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -397,9 +399,12 @@ public abstract class WebApplication
         // log source
         String origin = (source!=null ? source.getPageDefinition().getPageBeanName() : "[Unknown]");
         log.error("Fatal error of type {} from \"{}\": {}: {}", e.getClass().getName(), origin, e.getMessage());
+        
+        // check handled
+        boolean handled = (context.getMaximumSeverity()==FacesMessage.SEVERITY_ERROR);
 
         // For page errors, give the ExceptionHandler a chance to handle
-        if (source!=null)
+        if (!handled && source!=null)
         {
             // Queue event 
             ExceptionQueuedEventContext event = new ExceptionQueuedEventContext(context, e, null, context.getCurrentPhaseId());
@@ -411,9 +416,29 @@ public abstract class WebApplication
             if (context.getResponseComplete())
                 return;
         }
-            
-        // If all has failed, redirect to ContextPath (root)
-        redirectDirectly(context, StringUtils.EMPTY);
+        
+        // Find message
+        FacesMessage facesMsg = null;
+        Iterator<FacesMessage> messages = context.getMessages();
+        while (messages.hasNext())
+        {
+            FacesMessage msg = messages.next();
+            if (msg.getSeverity()==FacesMessage.SEVERITY_ERROR)
+            {   // found
+                facesMsg = msg;
+                break;
+            }
+        }
+        if (facesMsg==null)
+            facesMsg = getFacesErrorMessage(context, origin, e);
+        // Set Session Message
+        ExternalContext ec = context.getExternalContext();
+        ec.getSessionMap().put(Page.SESSION_MESSAGE, facesMsg);
+
+        // If parentPage is null then redirect to ContextPath (root)
+        PageDefinition parentPage = (source!=null ? source.getParentPage() : null);
+        String redirectUrl = (parentPage!=null ? parentPage.getOutcome().toString() : StringUtils.EMPTY);
+        redirectDirectly(context, redirectUrl);
     }
     
     /**
@@ -519,7 +544,108 @@ public abstract class WebApplication
         }
         return component;
     }
+    
+    /**
+     * Registers a ManagedBean for a particular scope
+     * @param beanClass the bean class to register
+     * @param scope the bean scope
+     */
+    public void registerManagedBean(Class<?> beanClass, BeanScope scope)
+    {
+        // check
+        if (beanClass==null || scope==null)
+            throw new InvalidArgumentException("beanClass|scope", null);
+        // detect name
+        String className = beanClass.getName();
+        int    nameIndex = className.lastIndexOf('.')+1; 
+        String beanName  = className.substring(nameIndex, nameIndex+1).toLowerCase() + className.substring(nameIndex+1); 
+        log.info("Registering Managed Bean \"{}\" on {} scope for class {}", beanName, scope, className);
+        getFacesImplementation().registerManagedBean(beanName, className, scope.name().toLowerCase());
+    }
 
+    /**
+     * Returns the bean for a particular scope
+     * @param context the FacesContext
+     * @param beanName the beanName
+     * @param scope the bean scope
+     * @return the bean instance
+     */
+    public Object getContextBean(FacesContext context, String beanName, BeanScope scope)
+    {
+        // check
+        if (context==null || beanName==null || scope==null)
+            throw new InvalidArgumentException("context|beanName|beanInstance|scope", null);
+        // scope 
+        ExternalContext ec = context.getExternalContext();
+        Map<String, Object> map;
+        switch(scope)
+        {
+            case Application: map = ec.getApplicationMap();break;
+            case Session:     map = ec.getSessionMap();break;
+            case View:        map = context.getViewRoot().getViewMap();break;
+            case Request:     map = ec.getRequestMap();break;
+            default:
+                throw new NotSupportedException(this, "getContextBean:"+scope.name());
+        }
+        return map.get(beanName);
+    }
+    
+    /**
+     * Sets a bean for a particular scope
+     * @param context the FacesContext
+     * @param beanName the beanName
+     * @param beanInstance the bean instance
+     * @param scope the bean scope
+     */
+    public void setContextBean(FacesContext context, String beanName, Object beanInstance, BeanScope scope)
+    {
+        // check
+        if (context==null || beanName==null || beanInstance==null || scope==null)
+            throw new InvalidArgumentException("context|beanName|beanInstance|scope", null);
+        // log
+        if (log.isInfoEnabled()) {
+            Class<?> beanClass = (beanInstance instanceof Class<?>) ? ((Class<?>)beanInstance) : beanInstance.getClass();
+            log.info("Adding Context Bean \"{}\" on {} scope of type {}", beanName, scope, beanClass.getName());
+        }
+        ExternalContext ec = context.getExternalContext();
+        Map<String, Object> map;
+        switch(scope)
+        {
+            case Application: map = ec.getApplicationMap();break;
+            case Session:     map = ec.getSessionMap();break;
+            case View:        map = context.getViewRoot().getViewMap();break;
+            case Request:     map = ec.getRequestMap();break;
+            default:
+                throw new NotSupportedException(this, "setContextBean:"+scope.name());
+        }
+        map.put(beanName, beanInstance);
+    }
+    
+    /**
+     * Sets a bean for a particular scope
+     * The beanName will be auto detected from the beanInstance
+     * @param context the FacesContext
+     * @param beanInstance the bean instance
+     * @param scope the bean scope
+     */
+    public void setContextBean(FacesContext context, Object beanInstance, BeanScope scope)
+    {
+        String className;
+        String beanName;
+        if (beanInstance instanceof Class<?>)
+        {   // class objects are taken literally
+            className = ((Class<?>)beanInstance).getName();
+            beanName  = className.substring(className.lastIndexOf('.')+1);
+        }
+        else
+        {   // detect beanname from class
+            className = beanInstance.getClass().getName();
+            int nameIndex = className.lastIndexOf('.')+1; 
+            beanName = className.substring(nameIndex, nameIndex+1).toLowerCase() + className.substring(nameIndex+1);
+        }
+        setContextBean(context, beanName, beanInstance, scope);
+    }
+    
     /**
      * returns the default input control type for a given data Type
      * @see org.apache.empire.jsf2.controls.InputControlManager
@@ -555,10 +681,16 @@ public abstract class WebApplication
         for (int i = 0; locales.hasNext(); i++)
         {
             Locale locale = locales.next();
-            textResolvers[i] = new ResourceTextResolver(ResourceBundle.getBundle(messageBundle, locale));
+            ResourceBundle bundle = ResourceBundle.getBundle(messageBundle, locale);
+            textResolvers[i] = createResourceTextResolver(bundle);
             log.info("added TextResolver for {} bundle='{}'", locale.getLanguage(), messageBundle);
         }
     }
+    
+    protected ResourceTextResolver createResourceTextResolver(ResourceBundle bundle)
+    {
+        return new ResourceTextResolver(bundle);
+    }
 
     /**
      * Obtains a JDBC-Connection from the connection pool
