@@ -38,13 +38,15 @@ import org.apache.empire.commons.ObjectUtils;
 import org.apache.empire.data.Column;
 import org.apache.empire.jsf2.controls.InputControl;
 import org.apache.empire.jsf2.utils.ControlRenderInfo;
+import org.apache.empire.jsf2.utils.StyleClass;
 import org.apache.empire.jsf2.utils.TagEncodingHelper;
 import org.apache.empire.jsf2.utils.TagEncodingHelperFactory;
 import org.apache.empire.jsf2.utils.TagStyleClass;
+import org.apache.empire.jsf2.utils.TagEncodingHelperFactory.TagEncodingHolder;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-public class ControlTag extends UIInput implements NamingContainer
+public class ControlTag extends UIInput implements NamingContainer, TagEncodingHolder
 {
     /**
      * ControlSeparatorComponent
@@ -224,7 +226,9 @@ public class ControlTag extends UIInput implements NamingContainer
             if (valInfo == null)
                 valInfo = helper.getValueInfo(context); // Oops, should not come here 
 
-            String styleClass = helper.getTagStyleClass(TagStyleClass.INPUT_DIS.get());
+            StyleClass styleClass = helper.getTagStyleClass();
+            styleClass.add(TagStyleClass.INPUT_DIS);
+
             String tooltip = helper.getValueTooltip(helper.getTagAttributeValue("title"));
 
             // render components
@@ -242,7 +246,6 @@ public class ControlTag extends UIInput implements NamingContainer
     protected InputControl            control                = null;
     protected InputControl.InputInfo  inpInfo                = null;
     protected ControlRenderInfo       renderInfo             = null;
-    private boolean                   creatingComponents     = false;
     protected boolean                 submittedValueDetected = false;
     protected Object                  submittedValue;
     protected boolean                 valueValidated         = false;
@@ -256,7 +259,14 @@ public class ControlTag extends UIInput implements NamingContainer
     @Override
     public String getFamily()
     {
-        return "javax.faces.NamingContainer";
+        // return UINamingContainer.COMPONENT_FAMILY;
+        return TagEncodingHelper.COMPONENT_FAMILY;
+    }
+
+    @Override
+    public TagEncodingHelper getEncodingHelper()
+    {
+        return helper;
     }
 
     protected void saveState()
@@ -313,14 +323,11 @@ public class ControlTag extends UIInput implements NamingContainer
         // check whether already set
         if (helper.hasComponentId())
             return;
-        /*
-         * Attention: Only works if FormGrid is a direct parent of the Control.
-         * Does not work, if other components are between the Control and the FormGrid.
-         */
-        this.renderInfo = helper.getControlRenderInfo();
-        if (this.renderInfo!=null && this.renderInfo.AUTO_CONTROL_ID!=null) {
+        // set autoId
+        String autoId = helper.getAutoControlId();
+        if (autoId!=null) {
             // Auto set component Id
-            setId(this.renderInfo.AUTO_CONTROL_ID.toString());
+            setId(autoId);
             log.debug("Auto-Setting compontent id for Control to {}", this.getId());
         }
     }
@@ -343,7 +350,8 @@ public class ControlTag extends UIInput implements NamingContainer
     public String getClientId(FacesContext context)
     {
         // Check if dynamic components are being created
-        if (this.treeClientId!=null && (this.creatingComponents || this.control!=null && this.control.isCreatingComponents()))
+        // Check for "this.creatingComponents" has been removed on 2025-03-18
+        if (this.treeClientId!=null && control!=null && control.isCreatingComponents())
         {   // return the original tree client id
             return treeClientId; 
         }
@@ -413,14 +421,9 @@ public class ControlTag extends UIInput implements NamingContainer
             if (getChildCount() > 0)
                 labelSepTag = (ControlSeparatorComponent) getChildren().get(0);
             if (labelSepTag == null)
-            {   try {
-                    creatingComponents = true;
-                    labelSepTag = new LabelSeparatorComponent();
-                    getChildren().add(labelSepTag);
-                    helper.resetComponentId(labelSepTag);
-                } finally {
-                    creatingComponents = false;
-                }
+            {   labelSepTag = new LabelSeparatorComponent();
+                getChildren().add(labelSepTag);
+                helper.resetComponentId(labelSepTag);
             }
             // encode
             labelSepTag.setRendered(this.controlVisible);
@@ -434,14 +437,9 @@ public class ControlTag extends UIInput implements NamingContainer
             if (getChildCount() > 1)
                 inputSepTag = (ControlSeparatorComponent) getChildren().get(1);
             if (inputSepTag == null)
-            {   try {
-                    creatingComponents = true;
-                    inputSepTag = new InputSeparatorComponent();
-                    getChildren().add(inputSepTag);
-                    helper.resetComponentId(inputSepTag);
-                } finally {
-                    creatingComponents = false;
-                }
+            {   inputSepTag = new InputSeparatorComponent();
+                getChildren().add(inputSepTag);
+                helper.resetComponentId(inputSepTag);
             }
             // encode
             inputSepTag.setRendered(this.controlVisible);
@@ -542,26 +540,21 @@ public class ControlTag extends UIInput implements NamingContainer
         }
         else
         {   // render components
-            try {
-                creatingComponents = true;
-                HtmlOutputLabel labelComponent = null;
-                if (parent.getChildCount()==0)
-                {   // Create components
-                    String forInput = (customInput ? helper.getTagAttributeString("for") : "*");
-                    // createLabelComponent 
-                    String styleClass = TagStyleClass.LABEL.get();
-                    labelComponent = helper.createLabelComponent(context, forInput, styleClass, null, getColon());
-                    parent.getChildren().add(0, labelComponent);
-                    helper.resetComponentId(labelComponent);
-                }
-                else if (this.controlVisible)
-                {
-                    labelComponent = (HtmlOutputLabel) parent.getChildren().get(0);
-                    // update
-                    helper.updateLabelComponent(context, labelComponent, null);
-                }
-            } finally {
-                creatingComponents = false;
+            HtmlOutputLabel labelComponent = null;
+            if (parent.getChildCount()==0)
+            {   // Create components
+                String forInput = (customInput ? helper.getTagAttributeString("for") : "*");
+                // createLabelComponent 
+                StyleClass styleClass = new StyleClass(TagStyleClass.LABEL);
+                labelComponent = helper.createLabelComponent(context, forInput, styleClass, null, getColon());
+                parent.getChildren().add(0, labelComponent);
+                helper.resetComponentId(labelComponent);
+            }
+            else if (this.controlVisible)
+            {
+                labelComponent = (HtmlOutputLabel) parent.getChildren().get(0);
+                // update
+                helper.updateLabelComponent(context, labelComponent, null);
             }
         }
     }
@@ -575,46 +568,40 @@ public class ControlTag extends UIInput implements NamingContainer
     protected void encodeInput(FacesContext context, UIComponent parent)
         throws IOException
     {
-        // render components
-        try {
-            creatingComponents = true;
-            // check children
-            int count = parent.getChildCount();
-            UIComponent valueComp = (count>0 ? parent.getChildren().get(count-1) : null);
-            boolean resetChildId = (count==0);
-            // continue
-            this.inpInfo = helper.getInputInfo(context);
-            // set required
-            super.setRequired(this.controlVisible && helper.isValueRequired());
-            // create Input Controls
-            // boolean recordReadOnly = helper.isRecordReadOnly();
-            if (count==0)
-            {   // Create components
-                control.createInput(parent, inpInfo, context);
-                // create Value Component
-                if (valueComp == null)
-                {   // create ValueOutputComponent
-                    valueComp = new ValueOutputComponent();
-                    parent.getChildren().add(valueComp);
-                }
-            }
-            else if (this.controlVisible)
-            {   // Update
-                control.updateInputState(parent, inpInfo, context, context.getCurrentPhaseId());
+        // check children
+        int count = parent.getChildCount();
+        UIComponent valueComp = (count>0 ? parent.getChildren().get(count-1) : null);
+        boolean resetChildId = (count==0);
+        // continue
+        this.inpInfo = helper.getInputInfo(context);
+        // set required
+        super.setRequired(this.controlVisible && helper.isValueRequired());
+        // create Input Controls
+        // boolean recordReadOnly = helper.isRecordReadOnly();
+        if (count==0)
+        {   // Create components
+            control.createInput(parent, inpInfo, context);
+            // create Value Component
+            if (valueComp == null)
+            {   // create ValueOutputComponent
+                valueComp = new ValueOutputComponent();
+                parent.getChildren().add(valueComp);
             }
+        }
+        else if (this.controlVisible)
+        {   // Update
+            control.updateInputState(parent, inpInfo, context, context.getCurrentPhaseId());
+        }
+        // set rendered
+        boolean renderValue = helper.isRenderValueComponent();
+        List<UIComponent> children = parent.getChildren();
+        for (UIComponent child : children)
+        {   // reset child id
+            if (resetChildId && child.getId()!=null)
+                child.setId(child.getId());
             // set rendered
-            boolean renderValue = helper.isRenderValueComponent();
-            List<UIComponent> children = parent.getChildren();
-            for (UIComponent child : children)
-            {   // reset child id
-                if (resetChildId && child.getId()!=null)
-                    child.setId(child.getId());
-                // set rendered
-                boolean valueOutput = (child instanceof ValueOutputComponent);
-                child.setRendered((valueOutput ? renderValue : !renderValue));
-            }
-        } finally {
-            creatingComponents = false;
+            boolean valueOutput = (child instanceof ValueOutputComponent);
+            child.setRendered((valueOutput ? renderValue : !renderValue));
         }
     }
     
