@@ -46,6 +46,7 @@ import org.apache.empire.jsf2.controls.InputControl.DisabledType;
 import org.apache.empire.jsf2.controls.InputControl.InputInfo;
 import org.apache.empire.jsf2.controls.InputControlManager;
 import org.apache.empire.jsf2.controls.SelectInputControl;
+import org.apache.empire.jsf2.utils.StyleClass;
 import org.apache.empire.jsf2.utils.TagEncodingHelper;
 import org.apache.empire.jsf2.utils.TagStyleClass;
 import org.slf4j.Logger;
@@ -117,9 +118,9 @@ public class SelectTag extends UIInput implements NamingContainer
         }
 
         @Override
-        public String getStyleClass(String addlStyle)
+        public StyleClass getStyleClass()
         {
-            return null;
+            return new StyleClass();
         }
 
         @Override
@@ -201,7 +202,8 @@ public class SelectTag extends UIInput implements NamingContainer
     @Override
     public String getFamily()
     {
-        return "javax.faces.NamingContainer";
+        // return UINamingContainer.COMPONENT_FAMILY;
+        return TagEncodingHelper.COMPONENT_FAMILY;
     }
     
     /**
@@ -243,11 +245,15 @@ public class SelectTag extends UIInput implements NamingContainer
             if (inputComponent instanceof UISelectOne)
             {
                 this.control = (SelectInputControl) InputControlManager.getControl(SelectInputControl.NAME);
+                UISelectOne selectComp = (UISelectOne) inputComponent;
                 // disabled
-                setInputDisabled((UISelectOne)inputComponent, isDisabled());
+                selectComp.setRequired(isRequired()); // !isAllowNull()
+                setInputDisabled(selectComp, isDisabled());
                 // Options (sync)
-                control.syncOptions((UISelectOne) inputComponent, textResolver, selectInputInfo);
-                setInputValue((UISelectOne) inputComponent);
+                control.syncOptions(selectComp, textResolver, selectInputInfo);
+                // style and value
+                setInputStyleClass(selectComp);
+                setInputValue(selectComp);
             }
             else
             { // Something's wrong here?
@@ -278,12 +284,12 @@ public class SelectTag extends UIInput implements NamingContainer
         // nothing submitted (AJAX part request, e.g. calendar component) or readonly (won't be set
         // in updateModel())?
         UIInput inputComponent = getInputComponent();
-        if (inputComponent == null)
+        if (inputComponent == null || isDisabled())
         {
             return;
         }
         // component itself already checked validity, was it successful?
-        if (!inputComponent.isValid() || isDisabled())
+        if (!inputComponent.isValid())
         {
             return;
         }
@@ -373,15 +379,34 @@ public class SelectTag extends UIInput implements NamingContainer
         // Options
         control.initOptions(input, textResolver, selectInputInfo);
         // disabled
-        boolean disabled = setInputDisabled(input, isDisabled());
-        control.addRemoveDisabledStyle(input, disabled);
+        setInputDisabled(input, isDisabled());
+        // set style class
+        setInputStyleClass(input);
         // input.setLabel(getLabelString());
-        // input.setRequired(col.isRequired() && !col.isAutoGenerated());
+        input.setRequired(isRequired()); // !isAllowNull()
         // input.setId(this.getId() + INPUT_SUFFIX);
         setInputValue(input);
         return input;
     }
 
+    protected final void setInputStyleClass(UISelectOne input)
+    {
+        // css style
+        StyleClass styleClass = new StyleClass(TagStyleClass.SELECT);
+        styleClass.add(getAttributes().get(InputControl.CSS_STYLE_CLASS));
+        // additional style classes
+        if (isDisabled())
+        {   // disabled
+            styleClass.add(TagStyleClass.INPUT_DIS);
+        }
+        else if (input.isRequired() || !isAllowNull())
+        {   // required
+            styleClass.add(TagStyleClass.INPUT_REQ);
+        }
+        // String cssStyle = TagEncodingHelper.assembleStyleClassString(TagStyleClass.SELECT.get(), null, null, userStyle);
+        input.getAttributes().put(InputControl.CSS_STYLE_CLASS, styleClass.build());
+    }
+
     protected void setInputValue(UISelectOne input)
     {
         Object value = getValue();
@@ -398,6 +423,7 @@ public class SelectTag extends UIInput implements NamingContainer
     protected void copyAttributes(UISelectOne input)
     {
         // set id
+        /* removed 2025-06-19 to avoid doubling of given id
         if (TagEncodingHelper.hasComponentId(this))
         {   // remove trailing underscore (workaround since parent and child may not have the same name)
             String inputId = this.getId();
@@ -408,19 +434,18 @@ public class SelectTag extends UIInput implements NamingContainer
             input.setId(inputId);
         }
         else
-        {   // always set to CompoentID
+        {   // set to CompoentID
             input.setId(SELECT_COMPONENT_ID);
         }
-        
+        */
+        // always set to CompoentID
+        input.setId(SELECT_COMPONENT_ID);
+        // the map
         Map<String, Object> tagMap = getAttributes();
         Map<String, Object> inputMap = input.getAttributes();
-        // css style
-        String userStyle = StringUtils.toString(tagMap.get(InputControl.CSS_STYLE_CLASS));
-        // String cssStyle = TagEncodingHelper.assembleStyleClassString(TagStyleClass.SELECT.get(), null, null, userStyle);
-        String cssStyle = TagStyleClass.SELECT.append(userStyle);
-        inputMap.put(InputControl.CSS_STYLE_CLASS, cssStyle);
         // other
         copyAttribute(inputMap, tagMap, "style");
+        copyAttribute(inputMap, tagMap, "label");
         copyAttribute(inputMap, tagMap, "tabindex");
         copyAttribute(inputMap, tagMap, "onchange");
     }
@@ -439,7 +464,7 @@ public class SelectTag extends UIInput implements NamingContainer
             aoh.updateAttachedObjects(this, context, null, inputComponent);
     }
     
-    protected boolean setInputDisabled(UISelectOne input, boolean disabled)
+    protected void setInputDisabled(UISelectOne input, boolean disabled)
     {
         if (input instanceof HtmlSelectOneMenu)
             ((HtmlSelectOneMenu)input).setDisabled(disabled);
@@ -447,7 +472,6 @@ public class SelectTag extends UIInput implements NamingContainer
             ((HtmlSelectOneListbox)input).setDisabled(disabled);
         else
             log.warn("Unable to set disabled attribute!");
-        return disabled;
     }
     
     protected void copyAttribute(Map<String, Object> inputMap, Map<String, Object> tagMap, String name)
