@@ -29,6 +29,7 @@ import javax.faces.event.PhaseId;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
+import org.apache.empire.jsf2.utils.StyleClass;
 
 public class TextAreaInputControl extends InputControl
 {
@@ -56,18 +57,21 @@ public class TextAreaInputControl extends InputControl
     }
 
     @Override
-    public void renderValue(UIComponent comp, String tagName, String styleClass, String tooltip, ValueInfo vi, FacesContext context)
+    public void renderValue(UIComponent comp, String tagName, StyleClass styleClass, String tooltip, ValueInfo vi, FacesContext context)
             throws IOException
     {
-        styleClass += " textarea";
-        // cols
-        int cols = getFormatInteger(vi, FORMAT_COLS, FORMAT_COLS_ATTRIBUTE);
-        if (cols>0)
-            styleClass += " cols-"+String.valueOf(cols);
-        // rows
-        int rows = getFormatInteger(vi, FORMAT_ROWS, FORMAT_ROWS_ATTRIBUTE);
-        if (rows>0)
-            styleClass += " rows-"+String.valueOf(rows);
+        if (styleClass!=null)
+        {   // Extend style class
+            styleClass.add("textarea");
+            // cols
+            int cols = getFormatInteger(vi, FORMAT_COLS, FORMAT_COLS_ATTRIBUTE);
+            if (cols>0)
+                styleClass.add("cols-"+String.valueOf(cols));
+            // rows
+            int rows = getFormatInteger(vi, FORMAT_ROWS, FORMAT_ROWS_ATTRIBUTE);
+            if (rows>0)
+                styleClass.add("rows-"+String.valueOf(rows));
+        }
         // render
         super.renderValue(comp, tagName, styleClass, tooltip, vi, context);
     }
@@ -97,7 +101,7 @@ public class TextAreaInputControl extends InputControl
         // add
         compList.add(input);
         // update
-        updateInputState(compList, ii, context, context.getCurrentPhaseId());
+        updateInputState(compList, ii, context, PhaseId.RENDER_RESPONSE);
     }
     
     @Override
@@ -117,8 +121,7 @@ public class TextAreaInputControl extends InputControl
         // Set Value
         if (phaseId==PhaseId.RENDER_RESPONSE)
         {   // style
-            addRemoveDisabledStyle(input, (disabled!=null && disabled!=DisabledType.NO));
-            addRemoveInvalidStyle(input, ii.hasError());
+            setInputStyleClass(input, ii);
             // set value
             setInputValue(input, ii);
         }    
