@@ -55,6 +55,7 @@ import org.apache.empire.exceptions.InvalidValueException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
 import org.apache.empire.jsf2.components.ControlTag;
 import org.apache.empire.jsf2.components.InputTag;
+import org.apache.empire.jsf2.utils.StyleClass;
 import org.apache.empire.jsf2.utils.TagStyleClass;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -223,7 +224,7 @@ public class TextInputControl extends InputControl
             compList.add(createUnitLabel(TagStyleClass.INPUT_HINT.get(), ii, hint));
         }
         // update
-        updateInputState(compList, ii, context, context.getCurrentPhaseId());
+        updateInputState(compList, ii, context, PhaseId.RENDER_RESPONSE);
     }
 
     @Override
@@ -235,34 +236,14 @@ public class TextInputControl extends InputControl
             throw new UnexpectedReturnValueException(comp.getClass().getName(), "compList.get(0)");
         }
         HtmlInputText input = (HtmlInputText) comp;
-        if (ii.isInsideUIData())
-        {   // always reset the style class inside UIData
-            String tagStyle = StringUtils.toString(ii.getAttribute(InputControl.CSS_STYLE_CLASS), null);
-            String cssStyle = getInputStyleClass(ii, tagStyle);
-            input.setStyleClass(cssStyle);
-            /*
-            String curStyle = input.getStyleClass();
-            if (curStyle==null || !curStyle.equals(cssStyle))
-            {   // log.info("{} ->  {} vs '{}'", ii.getColumn().getName(), inpStyle, reqStyle);
-                input.setStyleClass(cssStyle);
-            }
-            */
-        }
-        // required
-        addRemoveStyle(input, TagStyleClass.INPUT_REQ, ii.isRequired());
-        // modified
-        addRemoveStyle(input, TagStyleClass.INPUT_MOD, ii.isModified());
-
         // disabled
         DisabledType disabled = ii.getDisabled();
         input.setReadonly((disabled==DisabledType.READONLY));
         input.setDisabled((disabled==DisabledType.DISABLED));
-        
         // set value
         if (phaseId==PhaseId.RENDER_RESPONSE)
         {   // style
-            addRemoveDisabledStyle(input, (disabled!=null && disabled!=DisabledType.NO));
-            addRemoveInvalidStyle(input, ii.hasError());
+            setInputStyleClass(input, ii);
             // set value
             setInputValue(input, ii);
         }
@@ -286,12 +267,12 @@ public class TextInputControl extends InputControl
     }
 
     @Override
-    protected void setInputStyleClass(UIInput input, String cssStyleClass)
+    protected void setInputStyleClass(UIInput input, StyleClass styleClass)
     {
         if (input instanceof HtmlInputText)
-            ((HtmlInputText)input).setStyleClass(cssStyleClass);
+            ((HtmlInputText)input).setStyleClass(styleClass.build());
         else
-            super.setInputStyleClass(input, cssStyleClass);
+            super.setInputStyleClass(input, styleClass);
     }
 
     @Override
@@ -479,14 +460,15 @@ public class TextInputControl extends InputControl
     public void renderValue(Object value, ValueInfo vi, ResponseWriter writer)
         throws IOException
     {
-        String text = formatValue(value, vi);
+        boolean escapeHtml = !hasFormatOption(vi, "noescape"); 
+        String text = formatValue(value, vi, escapeHtml);
         if (StringUtils.isEmpty(text))
-        { // nothing
+        {   // nothing
             writer.append(HTML_EXPR_NBSP);
             return;
         }
         // append text
-        writer.append(text);
+        super.renderValue(value, vi, writer);
         // unit?
         String unit = getUnitString(vi);
         if (StringUtils.isNotEmpty(unit) && !hasFormatOption(vi, "nounit"))
@@ -496,56 +478,6 @@ public class TextInputControl extends InputControl
         }
     }
 
-    /*
-    @Override
-    public void renderInput(Response writer, ControlInfo ci)
-    {
-        HtmlTag input = writer.startTag("input");
-        input.addAttribute("type", "text");
-        input.addAttribute("id",    ci.getId());
-        input.addAttribute("class", ci.getCssClass());
-        input.addAttribute("style", ci.getCssStyle());
-        if (ci.getDisabled()==false)
-        {   // Name of the field
-            input.addAttribute("name", ci.getName());
-            // Get Max Length
-            int maxLength = getMaxInputLength(ci.getColumn());
-            if (maxLength>0)
-            {
-                input.addAttribute("maxlength", maxLength);
-                input.addAttribute("size", String.valueOf(Math.min(maxLength, ci.getHSize())));
-            }   
-        }
-        else
-        {   // Disabled text control
-            input.addAttribute("disabled");
-            // Get Max Length
-            int maxLength = getMaxInputLength(ci.getColumn());
-            if (maxLength>0)
-            {
-                input.addAttribute("size", String.valueOf(Math.min(maxLength, ci.getHSize())));
-            }   
-        }
-        // Value
-        input.addAttribute("value", formatValue(ci, ci.getDisabled()));
-        // Event Attributes
-        input.addAttribute("onclick",   ci.getOnclick());
-        input.addAttribute("onchange",  ci.getOnchange());
-        input.addAttribute("onfocus",   ci.getOnfocus());
-        input.addAttribute("onblur",    ci.getOnblur());
-        input.endTag();
-        // Add Unit
-        if (ci.getDisabled()==false)
-        {   
-            String unit = getUnitString(ci);
-            if (unit != null)
-            {   writer.print(" ");
-                writer.print(unit);
-            }
-        }
-    }
-    */
-
     // ------- Input Helpers -------
 
     protected int getMaxInputLength(InputInfo ii)
@@ -645,7 +577,7 @@ public class TextInputControl extends InputControl
             return NumberFormat.getNumberInstance(locale);
         // Column is supplied
         String type = StringUtils.valueOf(column.getAttribute(Column.COLATTR_NUMBER_TYPE));
-        boolean isInteger = "Integer".equalsIgnoreCase(type);
+        boolean isInteger = Column.NUMTYPE_INTEGER.equalsIgnoreCase(type);
         NumberFormat nf = (isInteger) ? NumberFormat.getIntegerInstance(locale)
                                       : NumberFormat.getNumberInstance(locale);
         // Groups Separator?
@@ -750,7 +682,7 @@ public class TextInputControl extends InputControl
             String numberType = StringUtils.toString(column.getAttribute(Column.COLATTR_NUMBER_TYPE));
             if (numberType != null)
             {
-                if (numberType.equalsIgnoreCase("Currency"))
+                if (Column.NUMTYPE_CURRENCY.equalsIgnoreCase(numberType))
                 {
                     String currencyCode = StringUtils.toString(column.getAttribute(Column.COLATTR_CURRENCY_CODE));
                     if (currencyCode != null)
@@ -765,7 +697,7 @@ public class TextInputControl extends InputControl
                         return currencyCode;
                     }
                 }
-                else if (numberType.equalsIgnoreCase("Percent"))
+                else if (Column.NUMTYPE_PERCENT.equalsIgnoreCase(numberType))
                 {
                     return "%";
                 }
