@@ -19,20 +19,26 @@
 package org.apache.empire.jsf2.pageelements;
 
 import java.util.ArrayList;
-import java.util.HashSet;
+import java.util.Collection;
 import java.util.List;
+import java.util.Set;
 
 import javax.faces.event.ActionEvent;
 
+import org.apache.empire.commons.ArraySet;
 import org.apache.empire.commons.ObjectUtils;
+import org.apache.empire.commons.StringUtils;
+import org.apache.empire.data.Column;
+import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.ObjectNotValidException;
+import org.apache.empire.jsf2.app.FacesUtils;
 import org.apache.empire.jsf2.pages.Page;
 import org.apache.empire.jsf2.pages.PageElement;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-public abstract class ListPageElement<T> extends PageElement
+public abstract class ListPageElement<T> extends PageElement<Page>
 {
     // *Deprecated* private static final long serialVersionUID = 1L;
 
@@ -42,9 +48,13 @@ public abstract class ListPageElement<T> extends PageElement
 
     protected List<T>           items            = null;
 
-    protected SelectionSet      selectedItems    = null;
+    protected SelectionSet      selectedItemKeys = null;
     
-    public static class SelectionSet extends HashSet<String>
+    /**
+     * SelectionSet
+     * Contains the key of all items implementing "SelectableItem"
+     */
+    public static class SelectionSet extends ArraySet<Object[]>
     {
         private static final long serialVersionUID = 1L;
         
@@ -77,20 +87,37 @@ public abstract class ListPageElement<T> extends PageElement
             return invertSelection;
         }
 
-        public boolean set(String e)
+        public boolean set(Object[] e)
         {
             clear();
             return super.add(e);
         }
 
         @Override
-        public boolean add(String e)
+        public boolean add(Object[] e)
         {
+            if (!singleSelection && isInvertSelection()) 
+            {   // invert
+                if (this.contains(e))
+                    put(e);
+                return true;
+            }
+            // normal
             if (singleSelection)
                 return false;
             return super.add(e);
         }
 
+        public void put(Object[] key)
+        {
+            fastAdd(key);
+        }
+
+        public void putAll(Collection<Object[]> list)
+        {
+            addAll(list);
+        }
+
         @Override
         public boolean remove(Object o)
         {
@@ -134,34 +161,45 @@ public abstract class ListPageElement<T> extends PageElement
     {
         // *Deprecated* private static final long serialVersionUID = 1L;
 
-        private SelectionSet      selectSet        = null;
+        private Object[] key;
+        private SelectionSet selectSet = null;
 
-        protected void setSelectMap(SelectionSet selectSet)
+        protected void initSelect(Object[] key, SelectionSet selectSet)
         {
+            // check params
+            if (key==null || selectSet==null)
+                throw new InvalidArgumentException("key|selectSet", null);
+            // init
+            this.key = key;
             this.selectSet = selectSet;
         }
 
         public boolean isSelected()
         {
-            String id = getIdParam();
-            return this.selectSet.contains(id);
+            if (this.key==null)
+                throw new ObjectNotValidException(this);
+            return this.selectSet.contains(key);
         }
 
         public void setSelected(boolean selected)
         {
-            String id = getIdParam();
+            if (this.key==null)
+                throw new ObjectNotValidException(this);
+            // select or unselect
             if (selected != this.selectSet.isInvertSelection())
             {
-                this.selectSet.add(id);
+                this.selectSet.add(key);
             }
             else
             {
-                this.selectSet.remove(id);
+                this.selectSet.remove(key);
             }
         }
 
-        public abstract String getIdParam();
-
+        public Object[] getKey()
+        {
+            return key;
+        }
     }
 
     /**
@@ -236,7 +274,7 @@ public abstract class ListPageElement<T> extends PageElement
             return this.pageSize;
         }
 
-        /*** sorting ***/
+        /* sorting */
 
         public String getSortColumnName()
         {
@@ -256,7 +294,7 @@ public abstract class ListPageElement<T> extends PageElement
             this.sortOrderChanged = true;
         }
 
-        public boolean getSortAscending()
+        public boolean isSortAscending()
         {
             return this.sortAscending;
         }
@@ -282,8 +320,32 @@ public abstract class ListPageElement<T> extends PageElement
         {
             this.sortOrderChanged = sortOrderChanged;
         }
+        
+        public void setSortColumn(ActionEvent ae)
+        {
+            // Get column
+            Object column = FacesUtils.getActionEventAttribute(ae, "item", Object.class);
+            if (column==null) {
+                log.warn("Unable to set sort column: Attribute \"item\" missing on command link component!");
+                return;
+            }
+            // Get columnName
+            String columnName;
+            if (column instanceof Column) {
+                columnName = ((Column)column).getName();
+            } else {
+                columnName = column.toString();
+            }
+            // Change Column or Sort Order
+            if (StringUtils.compareEqual(columnName, getSortColumnName(), true)) {
+                // change order only
+                setSortAscending(!isSortAscending());
+            } else {
+                setSortColumnName(columnName);
+            }
+        }
 
-        /*** pagination ***/
+        /* pagination */
 
         public int getPosition()
         {
@@ -318,6 +380,16 @@ public abstract class ListPageElement<T> extends PageElement
             return (this.pageSize > 0);
         }
 
+        public boolean isFirstPage()
+        {
+            return (isValid() && isAllowPagination() && this.position==0);
+        }
+
+        public boolean isLastPage()
+        {
+            return (isValid() && isAllowPagination() && this.position + this.pageSize >= this.itemCount);
+        }
+
         public boolean isHasNextPage()
         {
             if (isValid() && isAllowPagination())
@@ -334,6 +406,7 @@ public abstract class ListPageElement<T> extends PageElement
 
         /**
          * set the next x entries
+         * @param e the action event
          */
         public void nextPage(ActionEvent e)
         {
@@ -353,6 +426,7 @@ public abstract class ListPageElement<T> extends PageElement
 
         /**
          * set the prev x entries
+         * @param e the action event
          */
         public void prevPage(ActionEvent e)
         {
@@ -378,6 +452,46 @@ public abstract class ListPageElement<T> extends PageElement
             this.modified = true;
         }
 
+        /**
+         * set the next x entries
+         * @param e the action event
+         */
+        public void firstPage(ActionEvent e)
+        {
+            if (!isAllowPagination())
+            {
+                throw new NotSupportedException(this, "nextPage");
+            }
+            // Check
+            if (this.position==0)
+            {
+                return; // Already on first Page
+            }
+            // First page
+            this.position = 0;
+            this.modified = true;
+        }
+
+        /**
+         * set the next x entries
+         * @param e the action event
+         */
+        public void lastPage(ActionEvent e)
+        {
+            if (!isAllowPagination())
+            {
+                throw new NotSupportedException(this, "nextPage");
+            }
+            // Check 
+            if (this.position == this.itemCount - this.pageSize)
+            {
+                return; // Already on last page
+            }
+            // One page forward
+            this.position = this.itemCount - this.pageSize;
+            this.modified = true;
+        }
+
         public int getPageNumber()
         {
             if (!isAllowPagination())
@@ -388,6 +502,11 @@ public abstract class ListPageElement<T> extends PageElement
             int pageNumber = new Double(Math.ceil(((double) this.position / this.pageSize))).intValue() + 1;
             return pageNumber;
         }
+        
+        public void setPageNumber(int newPageNumber) 
+        {
+            setPosition((--newPageNumber) * this.pageSize);
+        }
 
         public int getPageCount()
         {
@@ -410,7 +529,7 @@ public abstract class ListPageElement<T> extends PageElement
         // selectable
         if (ListPageElement.isSelectableItem(beanClass))
         {
-            this.selectedItems = new SelectionSet();
+            this.selectedItemKeys = new SelectionSet();
         }
     }
 
@@ -428,7 +547,8 @@ public abstract class ListPageElement<T> extends PageElement
         return false;
     }
 
-    /** session scoped properties **/
+    /* session scoped properties */
+
     public abstract ListTableInfo getTableInfo();
     
     public List<T> getItems()
@@ -495,34 +615,34 @@ public abstract class ListPageElement<T> extends PageElement
 
     public void clearSelection()
     {
-        if (this.selectedItems != null)
+        if (this.selectedItemKeys != null)
         {
-            this.selectedItems.clear();
+            this.selectedItemKeys.clear();
         }
     }
 
     public boolean isHasSelection()
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             return false;
         // Has selected Items
-        return (selectedItems.size()>0 || selectedItems.isInvertSelection());
+        return (selectedItemKeys.size()>0 || selectedItemKeys.isInvertSelection());
     }
 
     public int getSelectedItemCount()
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             return 0;
         // Item count
-        return this.selectedItems.size();
+        return this.selectedItemKeys.size();
     }
 
     public List<T> getSelectedItems()
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             throw new NotSupportedException(this, "getSelectedItems");
         // find all items
-        List<T> selection = new ArrayList<T>(this.selectedItems.size());
+        List<T> selection = new ArrayList<T>(this.selectedItemKeys.size());
         for (T item : getItems())
         {
             if (((SelectableItem)item).isSelected())
@@ -530,72 +650,79 @@ public abstract class ListPageElement<T> extends PageElement
         }
         return selection;
     }
+
+    public Set<Object[]> getSelectedItemKeys()
+    {
+        return selectedItemKeys;
+    }
+
+    public void setSelectedItems(Set<Object[]> items)
+    {
+        if (selectedItemKeys == null)
+            throw new NotSupportedException(this, "setSelectedItems");
+        // Get the set
+        selectedItemKeys = new SelectionSet(items.size());
+        for (Object[] key : items)
+        {
+            if (key == null || key.length == 0)
+            {
+                log.warn("Cannot select Null-Object.");
+                continue;
+            }
+            selectedItemKeys.add(key);
+        }
+    }
     
     public boolean isInvertSelection()
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             return false;
         // Invert selection
-        return this.selectedItems.isInvertSelection();
+        return this.selectedItemKeys.isInvertSelection();
     }
 
     public void setInvertSelection(boolean invertSelection)
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             throw new NotSupportedException(this, "setInvertSelection");
         // Invert
-        this.selectedItems.setInvertSelection(invertSelection);
+        this.selectedItemKeys.setInvertSelection(invertSelection);
     }
     
     public boolean isSingleSelection()
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             return false;
         // Invert selection
-        return this.selectedItems.isSingleSelection();
+        return this.selectedItemKeys.isSingleSelection();
     }
 
     public void setSingleSelection(boolean singleSelection)
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             throw new NotSupportedException(this, "setSingleSelection");
         // Invert
-        this.selectedItems.setSingleSelection(singleSelection);
+        this.selectedItemKeys.setSingleSelection(singleSelection);
     }
 
     public void setSelection(SelectableItem item)
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             throw new NotSupportedException(this, "setInvertSelection");
         // Invert
         if (item!=null)
-            this.selectedItems.set(item.getIdParam());
+            this.selectedItemKeys.set(item.getKey());
         else
-            this.selectedItems.clear();
+            this.selectedItemKeys.clear();
     }
 
     public void setSelection(SelectableItem[] items)
     {
-        if (this.selectedItems==null)
+        if (this.selectedItemKeys==null)
             throw new NotSupportedException(this, "setInvertSelection");
         // Invert
-        this.selectedItems.clear();
+        this.selectedItemKeys.clear();
         for (SelectableItem item : items)
-            this.selectedItems.add(item.getIdParam());
-    }
-
-    protected void assignSelectionMap(List<?> items)
-    {
-        // Check selectable
-        if (this.selectedItems == null)
-            return;
-        // generate all
-        for (Object item : items)
-        {
-            if (item instanceof SelectableItem)
-            {
-                ((SelectableItem) item).setSelectMap(this.selectedItems);
-            }
-        }
+            this.selectedItemKeys.add(item.getKey());
     }
 }
