@@ -19,7 +19,10 @@
 package org.apache.empire.jsf2.utils;
 
 import java.io.IOException;
-import java.lang.reflect.InvocationTargetException;
+import java.text.NumberFormat;
+import java.time.LocalDate;
+import java.time.LocalDateTime;
+import java.util.Date;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Locale;
@@ -31,6 +34,7 @@ import javax.faces.component.NamingContainer;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIData;
 import javax.faces.component.UIInput;
+import javax.faces.component.UINamingContainer;
 import javax.faces.component.UIOutput;
 import javax.faces.component.html.HtmlOutputLabel;
 import javax.faces.component.html.HtmlOutputText;
@@ -38,11 +42,9 @@ import javax.faces.component.html.HtmlPanelGroup;
 import javax.faces.context.FacesContext;
 import javax.faces.context.ResponseWriter;
 
-import org.apache.commons.beanutils.BeanUtils;
-import org.apache.commons.beanutils.BeanUtilsBean;
-import org.apache.commons.beanutils.PropertyUtils;
-import org.apache.commons.beanutils.PropertyUtilsBean;
 import org.apache.empire.commons.Attributes;
+import org.apache.empire.commons.BeanPropertyUtils;
+import org.apache.empire.commons.DateUtils;
 import org.apache.empire.commons.ObjectUtils;
 import org.apache.empire.commons.Options;
 import org.apache.empire.commons.StringUtils;
@@ -50,7 +52,7 @@ import org.apache.empire.commons.Unwrappable;
 import org.apache.empire.data.Column;
 import org.apache.empire.data.ColumnExpr;
 import org.apache.empire.data.DataType;
-import org.apache.empire.data.EntityType;
+import org.apache.empire.data.Entity;
 import org.apache.empire.data.Record;
 import org.apache.empire.data.RecordData;
 import org.apache.empire.db.DBDatabase;
@@ -58,11 +60,10 @@ import org.apache.empire.db.DBRecordBase;
 import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.exceptions.FieldIllegalValueException;
 import org.apache.empire.db.exceptions.FieldNotNullException;
-import org.apache.empire.exceptions.BeanPropertyGetException;
-import org.apache.empire.exceptions.BeanPropertySetException;
 import org.apache.empire.exceptions.EmpireException;
 import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.InvalidPropertyException;
+import org.apache.empire.exceptions.ItemNotFoundException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.PropertyReadOnlyException;
 import org.apache.empire.jsf2.app.FacesUtils;
@@ -90,11 +91,11 @@ import org.slf4j.LoggerFactory;
  */
 public class TagEncodingHelper implements NamingContainer
 {
+    public static final String COMPONENT_FAMILY = UINamingContainer.COMPONENT_FAMILY;
+    
     public static final String FACES_ID_PREFIX = "j_id";  // Standard Faces ID Prefix
 
     public static boolean CSS_STYLE_USE_INPUT_TYPE_INSTEAD_OF_DATA_TYPE = true;
-
-    private final static String SPACE = " ";
     
     /**
      * ColumnExprWrapper
@@ -128,10 +129,10 @@ public class TagEncodingHelper implements NamingContainer
         }
         
         @Override
-        public EntityType getEntityType()
+        public Entity getEntity()
         {
             Column column = getUpdateColumn();
-            return (column!=null ? column.getEntityType() : null);
+            return (column!=null ? column.getEntity() : null);
         }
 
         @Override
@@ -164,6 +165,17 @@ public class TagEncodingHelper implements NamingContainer
             return expr.getUpdateColumn().getAttributes();
         }
 
+        @SuppressWarnings("unchecked")
+        @Override
+        public <T extends ColumnExpr> T setAttribute(String name, Object value)
+        {
+            if (expr instanceof Column)
+               ((Column)expr).setAttribute(name, value);
+            else
+                throw new NotSupportedException(expr, "setAttribute");
+            return (T)this;
+        }
+
         @Override
         public Options getOptions()
         {
@@ -283,10 +295,9 @@ public class TagEncodingHelper implements NamingContainer
         }
 
         @Override
-        public String getStyleClass(String addlStyle)
+        public StyleClass getStyleClass()
         {
-            String style = getTagStyleClass(addlStyle);
-            return style; 
+            return getTagStyleClass(); 
         }
 
         @Override
@@ -298,7 +309,7 @@ public class TagEncodingHelper implements NamingContainer
                 // Use Column default
                 if (format==null && hasColumn())
                 { // from column
-                    format = StringUtils.toString(column.getAttribute("format"));
+                    format = getColumnAttributeString("format");
                 }
                 if (format==null)
                     format = StringUtils.EMPTY;
@@ -698,7 +709,7 @@ public class TagEncodingHelper implements NamingContainer
         if (column==null)
             return StringUtils.NULL;
         // Find Entity
-        EntityType entity = column.getEntityType();
+        Entity entity = column.getEntity();
         if (entity!=null)
             return StringUtils.concat(entity.getEntityName(), "_", column.getName());
         // No Entity
@@ -748,6 +759,12 @@ public class TagEncodingHelper implements NamingContainer
         if (this.valueInfo!=null) 
            ((ValueInfoImpl)this.valueInfo).reset();
     }
+
+    public void resetColumn()
+    {
+        this.column = null;
+        reset();
+    }
     
     public Object findRecordComponent()
     {
@@ -1404,65 +1421,21 @@ public class TagEncodingHelper implements NamingContainer
 
     protected Object getBeanPropertyValue(Object bean, String property)
     {
-        try
-        { // Get Property Value
-            PropertyUtilsBean pub = BeanUtilsBean.getInstance().getPropertyUtils();
-            return pub.getSimpleProperty(bean, property);
-
-        }
-        catch (IllegalAccessException e)
-        {
-            log.error(bean.getClass().getName() + ": unable to get property '" + property + "'");
-            throw new BeanPropertyGetException(bean, property, e);
-        }
-        catch (InvocationTargetException e)
-        {
-            log.error(bean.getClass().getName() + ": unable to get property '" + property + "'");
-            throw new BeanPropertyGetException(bean, property, e);
-        }
-        catch (NoSuchMethodException e)
-        {
-            log.warn(bean.getClass().getName() + ": no getter available for property '" + property + "'");
-            throw new BeanPropertyGetException(bean, property, e);
-        }
+        return BeanPropertyUtils.getProperty(bean, property);
     }
 
     protected void setBeanPropertyValue(Object bean, String property, Object value)
     {
-        // Get Property Name
-        try
-        { // Get Property Value
-            if (ObjectUtils.isEmpty(value))
-                value = null;
-            // Set Property Value
-            if (value != null)
-            { // Bean utils will convert if necessary
-                BeanUtils.setProperty(bean, property, value);
-            }
+        // set Property 
+        if (ObjectUtils.isEmpty(value))
+            value = null;
+        // Set Property Value
+        if (!BeanPropertyUtils.setProperty(bean, property, value))
+        {   // Property has not been set
+            if (BeanPropertyUtils.hasProperty(bean, property, true)==0)
+                throw new PropertyReadOnlyException(property);
             else
-            { // Don't convert, just set
-                PropertyUtils.setProperty(bean, property, null);
-            }
-        }
-        catch (IllegalArgumentException e)
-        {
-            log.error(bean.getClass().getName() + ": invalid argument for property '" + property + "'");
-            throw new BeanPropertySetException(bean, property, e);
-        }
-        catch (IllegalAccessException e)
-        {
-            log.error(bean.getClass().getName() + ": unable to set property '" + property + "'");
-            throw new BeanPropertySetException(bean, property, e);
-        }
-        catch (InvocationTargetException e)
-        {
-            log.error(bean.getClass().getName() + ": unable to set property '" + property + "'");
-            throw new BeanPropertySetException(bean, property, e);
-        }
-        catch (NoSuchMethodException e)
-        {
-            log.error(bean.getClass().getName() + ": no setter available for property '" + property + "'");
-            throw new BeanPropertySetException(bean, property, e);
+                throw new ItemNotFoundException(property);
         }
     }
     
@@ -1522,7 +1495,7 @@ public class TagEncodingHelper implements NamingContainer
                 if (options!=null && !hasFormat("notitlelookup"))
                     value = options.get(value);
                 // convert to display text
-                return getDisplayText(value);
+                return getDisplayText(ttc, value);
             } 
             else
             {   // Error
@@ -1541,11 +1514,11 @@ public class TagEncodingHelper implements NamingContainer
         if (!hasFormat("notitlelookup") && (options=getValueOptions())!=null)
         { // Lookup the title
             String optValue = options.get(value);
-            text = getDisplayText(optValue);
+            text = getDisplayText(column, optValue);
         }
         else
         {   // resolveText
-            text = getDisplayText(value);
+            text = getDisplayText(column, value);
         }
         // Check for template
         if (valIndex >= 0 && text!=null)
@@ -1557,13 +1530,12 @@ public class TagEncodingHelper implements NamingContainer
     {
         String title = getTagAttributeString("title");
         if (title == null)
-            title = StringUtils.toString(column.getAttribute(Column.COLATTR_TOOLTIP));
+            title = getColumnAttributeString(Column.COLATTR_TOOLTIP);
         if (title != null)
-            return getDisplayText(title);
+            return getDisplayText(column, title);
         // Check for short form
-        if (hasFormat("short") && !ObjectUtils.isEmpty(column.getAttribute(COLATTR_ABBR_TITLE)))
-            return getDisplayText(column.getTitle());
-        
+        if (hasFormat("short") && !ObjectUtils.isEmpty(getColumnAttribute(COLATTR_ABBR_TITLE)))
+            return getDisplayText(column, column.getTitle());
         // No Title
         return null;
     }
@@ -1580,18 +1552,47 @@ public class TagEncodingHelper implements NamingContainer
         return (f != null && f.indexOf(format) >= 0);
     }
     
-    public String getDisplayText(Object value)
+    public String getDisplayText(Column column, Object value)
     {
         if (ObjectUtils.isEmpty(value))
             return null;
-        if (value instanceof Number)
-            return String.valueOf(value);
+        // number
+        if (value instanceof Number) {
+            boolean isDecimal = (column!=null && (column.getDataType()==DataType.DECIMAL || column.getDataType()==DataType.FLOAT));
+            if (isDecimal) {
+                NumberFormat nf = NumberFormat.getIntegerInstance(getLocale());
+                return nf.format(value);
+            }
+            else
+                return String.valueOf(value);
+        }
+        // format date 
+        if (value instanceof Date) {
+            String date = DateUtils.formatDate((Date)value, getLocale());
+            if (column!=null && column.getDataType()!=DataType.DATE)
+                return StringUtils.concat(date, " ", DateUtils.formatTime((Date)value, getLocale(), (column.getDataType()==DataType.TIMESTAMP)));
+            else
+                return date;
+        }
+        if (value instanceof LocalDate)
+            return DateUtils.formatDate((LocalDate)value, getLocale());
+        if (value instanceof LocalDateTime) {
+            if (column!=null && column.getDataType()!=DataType.DATE)
+                return DateUtils.formatDateTime((LocalDateTime)value, getLocale(), (column.getDataType()==DataType.TIMESTAMP));
+            else
+                return DateUtils.formatDate((LocalDateTime)value, getLocale());
+        }
         // Resolve text
         if (textResolver==null)
             getTextResolver(FacesContext.getCurrentInstance());
         return textResolver.resolveText(value.toString());
     }
 
+    public final String getDisplayText(Object value)
+    {
+        return getDisplayText(this.column, value);
+    }
+    
     public TextResolver getTextResolver(FacesContext context)
     {
         if (textResolver==null)
@@ -1599,6 +1600,13 @@ public class TagEncodingHelper implements NamingContainer
         return textResolver;
     }
     
+    public Locale getLocale()
+    {
+        if (textResolver==null)
+            getTextResolver(FacesContext.getCurrentInstance());
+        return textResolver.getLocale();
+    }
+    
     /* ********************** Error messages ********************** */
     
     protected boolean detectError(FacesContext context)
@@ -1653,6 +1661,22 @@ public class TagEncodingHelper implements NamingContainer
         return (formGrid!=null) ? formGrid.getControlRenderInfo() : null;  
     }
     
+    public String getAutoControlId()
+    {
+        /*
+         * Attention: renderInfo may only be valid if FormGrid is a direct parent of the Control.
+         * If another component is between the Control and the FormGrid, then the "autoControlId" attribute must be set directly on the parent
+         */
+        ControlRenderInfo renderInfo = getControlRenderInfo();
+        if (renderInfo==null)
+            return null;
+        // check AUTO_CONTROL_ID
+        if (renderInfo.AUTO_CONTROL_ID==null)
+            return null;
+        // use AUTO_CONTROL_ID placeholder (*,$,@) 
+        return renderInfo.AUTO_CONTROL_ID.toString();
+    }
+    
     public String getControlExtraLabelWrapperStyle()
     {
         if (!ControlRenderInfo.isRenderExtraWrapperStyles())
@@ -1666,6 +1690,9 @@ public class TagEncodingHelper implements NamingContainer
     {
         if (!ControlRenderInfo.isRenderExtraWrapperStyles())
             return null;
+        /*
+         * Discarded due to missing use-case 
+         * 
         // input Wrapper Class
         String inputClass = (isControlTagElementValid() ? getTagAttributeStringEx(InputControl.CSS_STYLE_CLASS, true) : getTagAttributeString("inputClass")); 
         // append input state
@@ -1673,6 +1700,8 @@ public class TagEncodingHelper implements NamingContainer
             inputClass = (inputClass!=null ? StringUtils.concat(inputClass, " ", TagStyleClass.INPUT_DIS.get()) : TagStyleClass.INPUT_DIS.get());
         // done
         return inputClass;
+        */
+        return null;
     }
     
     /**
@@ -1696,7 +1725,7 @@ public class TagEncodingHelper implements NamingContainer
         {   // Check for short form    
             if (hasFormat("short"))
             {
-                label = StringUtils.toString(column.getAttribute(COLATTR_ABBR_TITLE));
+                label = getColumnAttributeString(COLATTR_ABBR_TITLE);
                 if (label==null)
                     log.warn("No Abbreviation available for column {}. Using normal title.", column.getName());
             }
@@ -1704,7 +1733,7 @@ public class TagEncodingHelper implements NamingContainer
             if (label==null)
                 label=column.getTitle();
             // translate
-            label = getDisplayText(label);
+            label = getDisplayText(column, label);
         }    
         // handle empty string
         if (StringUtils.isEmpty(label))
@@ -1716,7 +1745,7 @@ public class TagEncodingHelper implements NamingContainer
         return label;
     }
     
-    public HtmlOutputLabel createLabelComponent(FacesContext context, String forInput, String styleClass, String style, boolean colon)
+    public HtmlOutputLabel createLabelComponent(FacesContext context, String forInput, StyleClass styleClass, String style, boolean colon)
     {
         // forInput provided
         if (StringUtils.isNotEmpty(forInput))
@@ -1761,9 +1790,9 @@ public class TagEncodingHelper implements NamingContainer
         if (!ControlRenderInfo.isRenderExtraWrapperStyles())
             labelClass = getTagAttributeStringEx("labelClass", true);
         if (labelClass!=null)
-            styleClass = assembleStyleClassString(styleClass, labelClass);
-        if (StringUtils.isNotEmpty(styleClass))
-            label.setStyleClass(completeLabelStyleClass(styleClass, isValueRequired()));
+            styleClass.add(labelClass);
+        if (styleClass.isNotEmpty())
+            label.setStyleClass(completeLabelStyleClass(styleClass.build(), isValueRequired()));
         
         // for 
         if (StringUtils.isNotEmpty(forInput) && !isReadOnly())
@@ -1839,7 +1868,7 @@ public class TagEncodingHelper implements NamingContainer
 
     protected String completeLabelStyleClass(String styleClasses, boolean required)
     {
-        styleClasses = TagStyleClass.INPUT_REQ.addOrRemove(styleClasses, required);
+        styleClasses = StyleClass.addOrRemove(styleClasses, TagStyleClass.INPUT_REQ.get(), required);
         return styleClasses;
     }
     
@@ -1855,47 +1884,6 @@ public class TagEncodingHelper implements NamingContainer
     
     /* ********************** CSS-generation ********************** */
 
-    protected String assembleStyleClassString(String... styles)
-    {
-        int totalLength=0;
-        String current = null;
-        // Count total length
-        for (int i=0; i<styles.length; i++)
-        {   int len = (styles[i]!=null ? styles[i].length() : 0);
-            if (len>0) {
-                if (current!=null && current.equals(styles[i]))
-                    continue; // same style twice
-                current = styles[i];
-                if (totalLength>0 && !current.startsWith(SPACE))
-                    len++;
-                totalLength += len;
-            }
-        }
-        // only one?
-        if (current==null || current.length()==totalLength)
-            return current;
-        // concat now
-        current = null;
-        StringBuilder b = new StringBuilder(totalLength);
-        for (int i=0; i<styles.length; i++)
-        {   int len = (styles[i]!=null ? styles[i].length() : 0);
-            if (len>0) {
-                if (current!=null && current.equals(styles[i]))
-                    continue; // same style twice
-                current = styles[i];
-                if (b.length()>0 && !current.startsWith(SPACE))
-                    b.append(SPACE);
-                b.append(current);
-            }
-        }
-        /* can never be
-        if (b.length()!=totalLength)
-            log.warn("Wrong calculation for assembleStyleClassString!");
-        */    
-        // done
-        return b.toString();
-    }
-
     public static final String CSS_DATA_TYPE_NONE     = null;
     public static final String CSS_DATA_TYPE_INT      = " eTypeInt";
     public static final String CSS_DATA_TYPE_NUMBER   = " eTypeNumber";
@@ -1938,34 +1926,35 @@ public class TagEncodingHelper implements NamingContainer
         }
     }
 
-    public String getSimpleStyleClass(String userStyle)
+    public StyleClass getSimpleStyleClass(String userStyle)
     {
-        return assembleStyleClassString(cssStyleClass, userStyle);
+        StyleClass styleClass = new StyleClass(cssStyleClass);
+        return styleClass.add(userStyle);
     }
 
-    public String getSimpleStyleClass()
+    public StyleClass getSimpleStyleClass()
     {
         String userStyle = getTagAttributeString(InputControl.CSS_STYLE_CLASS);
         return getSimpleStyleClass(userStyle);
     }
 
-    public String getTagStyleClass(String typeClass, String addlStyle, String userStyle)
+    public StyleClass getTagStyleClass(String typeClass, String userStyle)
     {
-        // contextStyle
-        String contextStyle = getContextStyleClass();
-        return assembleStyleClassString(cssStyleClass, typeClass, addlStyle, userStyle, contextStyle);
+        StyleClass styleClass = new StyleClass(cssStyleClass);
+        styleClass.add(typeClass, userStyle, getContextStyleClass());
+        return styleClass;
     }
 
-    public String getTagStyleClass(String typeClass, String addlStyle)
+    public StyleClass getTagStyleClass(String typeClass)
     {
         String userStyle = getTagAttributeStringEx(InputControl.CSS_STYLE_CLASS, true);
-        return getTagStyleClass(typeClass, addlStyle, userStyle);
+        return getTagStyleClass(typeClass, userStyle);
     }
 
-    public String getTagStyleClass(String addlStyle)
+    public StyleClass getTagStyleClass()
     {
         String typeClass = (this.control!=null ? this.control.getCssStyleClass() : null);
-        return getTagStyleClass(typeClass, addlStyle);
+        return getTagStyleClass(typeClass);
     }
     
     public String getContextStyleClass()
@@ -2072,6 +2061,16 @@ public class TagEncodingHelper implements NamingContainer
     }
 
     /* ********************** Attribute value ********************** */
+
+    public Object getColumnAttribute(String name)
+    {
+        return (column!=null ? column.getAttribute(name) : null);
+    }
+
+    public String getColumnAttributeString(String name)
+    {
+        return StringUtils.nullIfEmpty(getColumnAttribute(name));
+    }
     
     public Object getTagAttributeValue(String name)
     {
@@ -2128,7 +2127,7 @@ public class TagEncodingHelper implements NamingContainer
         // Get column attribute
         if ((value==null || append) && hasColumn())
         {   // Check Column
-            Object colValue = column.getAttribute(name);
+            Object colValue = getColumnAttribute(name);
             if (append) 
             {   // append styles
                 if (ObjectUtils.isNotEmpty(colValue) && !colValue.equals(value))
@@ -2152,11 +2151,10 @@ public class TagEncodingHelper implements NamingContainer
                 String property = col.getBeanPropertyName();
                 try
                 {   // Use Beanutils to get Property
-                    PropertyUtilsBean pub = BeanUtilsBean.getInstance().getPropertyUtils();
-                    return pub.getSimpleProperty(rec, property);
+                    return BeanPropertyUtils.getProperty(rec, property);
                 }
                 catch (Exception e)
-                {   log.error("BeanUtils.getSimpleProperty failed for "+property, e);
+                {   log.error("BeanUtils.getProperty failed for property {} on {} ", property, rec.getClass().getName(), e);
                     return null;
                 }
             }    
@@ -2236,6 +2234,8 @@ public class TagEncodingHelper implements NamingContainer
         String wrapperClass = getTagAttributeStringEx("wrapperClass", true); 
         if (wrapperClass==null || wrapperClass.equals("-"))
             return null;
+        if (wrapperClass.equals("*"))
+            wrapperClass=null; // only use default style class
         // start element
         String tagName = InputControl.HTML_TAG_DIV;
         ResponseWriter writer = context.getResponseWriter();
