@@ -187,13 +187,12 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         }
     }
     
-    public Options(Class<?> enumType)
+    public Options(Class<? extends Enum<?>> enumType)
     {   // must be an enum
-        if (enumType==null || !enumType.isEnum())
+        if (enumType==null)
             throw new InvalidArgumentException("enumType", enumType);
         // create options from enum
-        @SuppressWarnings("unchecked")
-        Enum<?>[] items = ((Class<Enum<?>>)enumType).getEnumConstants();
+        Enum<?>[] items = enumType.getEnumConstants();
         this.list = new ArrayList<OptionEntry>(items.length);
         for (int i=0; i<items.length; i++)
         {
@@ -284,6 +283,18 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         return (oe!=null ? oe.isActive() : false);
     }
 
+    /**
+     * Gets the entry at index i
+     * 
+     * @param i the index
+     * 
+     * @return the OptionEntry or null
+     */
+    public OptionEntry getEntryAt(int i)
+    {
+        return (i>=0 && i<list.size() ? list.get(i) : null);
+    }
+
     /**
      * Gets the value of the entry at index i
      * 
