@@ -20,18 +20,21 @@ package org.apache.empire.db;
 
 import java.util.Collections;
 import java.util.Set;
+import java.util.regex.Pattern;
 
 import org.apache.empire.commons.Attributes;
 import org.apache.empire.commons.Options;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.Column;
 import org.apache.empire.data.DataType;
-import org.apache.empire.data.EntityType;
+import org.apache.empire.data.Entity;
 import org.apache.empire.db.exceptions.DatabaseNotOpenException;
 import org.apache.empire.db.expr.column.DBAliasExpr;
-import org.apache.empire.db.expr.column.DBValueExpr;
+import org.apache.empire.db.expr.column.DBFuncExpr;
 import org.apache.empire.db.expr.set.DBSetExpr;
 import org.apache.empire.dbms.DBMSHandler;
+import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.ObjectNotValidException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -62,7 +65,7 @@ public abstract class DBColumn extends DBColumnExpr
     protected final String    name;
     protected String          comment;
 
-    private Boolean quoteName = null;
+    protected Boolean quoteName = null;
     
     /**
      * Constructs a DBColumn object and set the specified parameters to this object.
@@ -213,7 +216,7 @@ public abstract class DBColumn extends DBColumnExpr
      */
     @Override
     public abstract Element addXml(Element parent, long flags);
-
+    
     /**
      * @return the current DBDatabase object
      */
@@ -254,6 +257,9 @@ public abstract class DBColumn extends DBColumnExpr
         if (dbms==null)
         	throw new DatabaseNotOpenException(getDatabase());
         // Append the name
+        if (quoteName==null)
+            quoteName=dbms.detectQuoteName(this, name);
+        // append
         dbms.appendObjectName(sql, name, quoteName);
     }
 
@@ -284,7 +290,7 @@ public abstract class DBColumn extends DBColumnExpr
      * Same as getRowSet()
      */
     @Override
-    public EntityType getEntityType()
+    public Entity getEntity()
     {
         return rowset;
     }
@@ -316,7 +322,7 @@ public abstract class DBColumn extends DBColumnExpr
      */
     public DBColumnExpr reference()
     {
-        return new DBValueExpr(getDatabase(), this, DataType.UNKNOWN);    
+        return getDatabase().getValueExpr(this, DataType.UNKNOWN);    
     }
 
     /**
@@ -351,7 +357,7 @@ public abstract class DBColumn extends DBColumnExpr
      * @return a qualified expression for this column
      */
     @Override
-    public DBColumnExpr qualified()
+    public DBAliasExpr qualified()
     {
         return this.as(getAlias());
     }
@@ -362,7 +368,7 @@ public abstract class DBColumn extends DBColumnExpr
      * @param nullValue the alternative value when this column is null
      * @return the coalesce and alias expression
      */
-    public DBColumnExpr coalesceColumn(Object nullValue)
+    public DBAliasExpr coalesceColumn(Object nullValue)
     {
         return coalesce(nullValue).as(this);
     }
@@ -388,8 +394,9 @@ public abstract class DBColumn extends DBColumnExpr
                                  : Collections.EMPTY_SET);
     }
 
-    /**
-     *  @see DBColumnExpr#getOptions()
+    /*
+     * (non-Javadoc)
+     * @see org.apache.empire.db.DBColumnExpr#getOptions()
      */
     @Override
     public Options getOptions()
@@ -399,26 +406,28 @@ public abstract class DBColumn extends DBColumnExpr
 
     /**
      * Returns true if an enum type has been set for this column
-     * <P>
      * @return true if an enum type has been set for this column or false otherwise
      */
     public final boolean isEnum()
     {
         return (getEnumType()!=null);
     }
-
+    
     /**
      * Returns the enum type for this column
-     * <P>
      * @return the enum type
      */
     @Override
     @SuppressWarnings("unchecked")
     public Class<Enum<?>> getEnumType()
     {
-        return (attributes!=null ? (Class<Enum<?>>)getAttribute(COLATTR_ENUMTYPE) : null);
+        return (Class<Enum<?>>)getAttribute(COLATTR_ENUMTYPE);
     }
     
+    /**
+     * Returns the java type for this column
+     * @return the java type
+     */
     @Override
     public Class<?> getJavaType()
     {
@@ -428,6 +437,131 @@ public abstract class DBColumn extends DBColumnExpr
         // default
         return super.getJavaType();
     }
+    
+    /**
+     * Sets the case sensitivity of the columnExpr
+     * @param caseSensitiv the character casing. This may be true, false or null (default)
+     * @return self (this)
+     */
+    @SuppressWarnings("unchecked")
+    public <T extends DBColumn> T setCaseSensitive(Boolean caseSensitiv)
+    {
+        if (!getDataType().isText())
+            throw new NotSupportedException(this, "setCaseInsensitive");
+        // set now
+        setAttribute(Column.COLATTR_CASESENSITIVE, caseSensitiv);
+        return (T)this;
+    }
+    
+    /**
+     * Sets the case sensitivity of the column to insensitive
+     * Text columns are case sensitive by default
+     * @return self (this)
+     */
+    public final <T extends DBColumn> T setCaseInsensitive()
+    {
+        return setCaseSensitive(false);
+    }
+
+    /**
+     * Sets the number type to a given value
+     * @param numberType the number type. See Column.NUMTYPE_... constants
+     * @return self (this)
+     */
+    @SuppressWarnings("unchecked")
+    public final <T extends DBColumn> T setNumberType(String numberType)
+    {
+        // only for text expressions
+        if (!getDataType().isNumeric())
+            throw new NotSupportedException(this, "setNumberType");
+        // set now
+        setAttribute(Column.COLATTR_NUMBER_TYPE, numberType);
+        return (T)this;
+    }
+
+    /**
+     * Returns the maximum allowed value 
+     * @return the minimum length 
+     */
+    public Pattern getRegExPattern()
+    {
+        return (Pattern)getAttribute(Column.COLATTR_REGEXP);
+    }
+    
+    /**
+     * Set the regular expression to validate the columnExpr value 
+     * @param regex the regular expression
+     * @return self (this)
+     */
+    @SuppressWarnings("unchecked")
+    public <T extends DBColumn> T setRegExPattern(String regex)
+    {
+        setAttribute(Column.COLATTR_REGEXP, Pattern.compile(regex));
+        return (T)this;
+    }
+    
+    /**
+     * Returns the normalized column for the columnExpr (if any)
+     * @return the normalized column or null
+     */
+    public DBColumnExpr getNormalizedColumn()
+    {   // return attribute
+        return (DBColumn)getAttribute(Column.COLATTR_NORMCOLUMN);
+    }
+    
+    /**
+     * Sets a normalized columnExpr for this columnExpr
+     * @param normalizedColumn the normalized columnExpr
+     * @return self (this)
+     */
+    @SuppressWarnings("unchecked")
+    public <T extends Column> T setNormalizedColumn(DBColumnExpr normalizedColumn)
+    { 
+        setAttribute(Column.COLATTR_NORMCOLUMN, normalizedColumn);
+        return (T)this;
+    }
+    
+    /**
+     * Returns the sort expression for a given column
+     * If no sort expression is explicitly set then the column itself is returned
+     * The returned expression should be assigned to an DBCommand.orderBy() function.
+     * 
+     * @return the sort expression or the column itself if not sort expression is set
+     */
+    public DBColumnExpr getSortExpr()
+    {
+        Object value = getAttribute(Column.COLATTR_SORTEXPRESSION);
+        if (value==null)
+        {   // not set
+            return this;
+        }
+        else if (value instanceof DBColumnExpr)
+        {   // return expression
+            return ((DBColumnExpr)value); 
+        }
+        else if ((value instanceof String) && ((String)value).indexOf('?')>=0)
+        {   // create a sort function expression
+            String sortFunctionTemplate = StringUtils.toString(value);
+            return new DBFuncExpr(this, sortFunctionTemplate, null, false, getDataType());
+        }
+        else
+        {   // unknown value
+            log.warn("Invalid value for {}: {}", Column.COLATTR_SORTEXPRESSION, value);
+            return this; 
+        }
+    }
+    
+    /**
+     * Sets a sort function expression for a given column
+     * @param sortExpression the expression which to use for sorting
+     * @return self (this)
+     */
+    @SuppressWarnings("unchecked")
+    public <T extends DBColumn> T setSortExpr(DBColumnExpr sortExpression)
+    {   // set sort expression
+        setAttribute(Column.COLATTR_SORTEXPRESSION, sortExpression);
+        return (T)this;
+    }
 
     /**
      * Creates and returns a sql-expression that maps enum values by name or ordinal to their string representation 
@@ -454,6 +588,22 @@ public abstract class DBColumn extends DBColumnExpr
         return super.decodeSort(getEnumType(), defaultToEnd);
     }
     
+    /**
+     * Sets a sort function template for a given column
+     * @param sortFunctionTemplate the template which must contain a ? which will be replaced with the column name
+     * @return return the column 
+     */
+    @SuppressWarnings("unchecked")
+    public <T extends DBColumn> T setSortExpr(String sortFunctionTemplate)
+    {
+        // check param
+        if (sortFunctionTemplate!=null && sortFunctionTemplate.indexOf('?')<0)
+            throw new InvalidArgumentException("sortFunctionTemplate", sortFunctionTemplate);
+        // set sort expression
+        setAttribute(Column.COLATTR_SORTEXPRESSION, sortFunctionTemplate);
+        return (T)this;
+    }
+    
     /**
      * Creates and returns a new DBSetExpr object.
      *
