@@ -31,6 +31,7 @@ import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.DataType;
 import org.apache.empire.db.DBIndex.DBIndexType;
 import org.apache.empire.db.DBRelation.DBCascadeAction;
+import org.apache.empire.db.DBRelation.DBReference;
 import org.apache.empire.db.exceptions.NoPrimaryKeyException;
 import org.apache.empire.db.exceptions.RecordDeleteFailedException;
 import org.apache.empire.db.exceptions.RecordUpdateFailedException;
@@ -81,6 +82,7 @@ public class DBTable extends DBRowSet implements Cloneable
      * 
      * @param name the table name
      * @param db the valid database object
+     * @param alias the table alias
      */
     public DBTable(String name, DBDatabase db, String alias)
     { 
@@ -187,7 +189,7 @@ public class DBTable extends DBRowSet implements Cloneable
         // set new alias
         clone.alias = generateAlias("t");
         // done
-        log.info("clone: Table " + name + " cloned! Alias old=" + alias + " new=" + clone.alias);
+        log.info("clone: Table {} cloned! Alias old={} new={}", name, alias, clone.alias);
         db.addTable(clone);
         return clone;
     }
@@ -206,12 +208,12 @@ public class DBTable extends DBRowSet implements Cloneable
             else
                 clone.alias = newAlias;
             // done
-            log.info("clone: Table " + name + " cloned! Alias old=" + alias + " new=" + clone.alias);
+            log.info("clone: Table {} cloned! Alias old={} new={}", name, alias, clone.alias);
             db.addTable(clone);
             return (T)clone;
         } catch (CloneNotSupportedException e) {
             // unable to clone table
-            log.error("Unable to clone table " + getName());
+            log.error("Unable to clone table {}", getName());
             throw new RuntimeException(e);
         }
     }
@@ -265,7 +267,7 @@ public class DBTable extends DBRowSet implements Cloneable
         if (getColumn(columnName)!=null)
             throw new ItemExistsException(columnName);
         // Make sure (DataType.INTEGER & DataMode.AutoGenerated) = DataType.AUTOINC
-        boolean autoGenerated = (type==DataType.AUTOINC || type==DataType.UNIQUEID);
+        boolean autoGenerated = (type==DataType.AUTOINC || type==DataType.UNIQUEID || (defValue instanceof DBGeneratedValue));
         DBTableColumn column = new DBTableColumn(this, type, columnName, size, required, autoGenerated, defValue);
         // auto-set primary key
         if (column.getDataType()==DataType.AUTOINC)
@@ -370,7 +372,7 @@ public class DBTable extends DBRowSet implements Cloneable
      * @param enumType  the class of the enum type
      * @return the new column object 
      */
-    public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Class<?> enumType)
+    public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Class<? extends Enum<?>> enumType)
     {
         if (!enumType.isEnum())
         {   // Class must be an enum type
@@ -393,12 +395,13 @@ public class DBTable extends DBRowSet implements Cloneable
      * @param enumValue the default value
      * @return the new column object 
      */
+    @SuppressWarnings("unchecked")
     public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Enum<?> enumValue)
     { 
         Object defValue = ObjectUtils.convertValue(type, enumValue);
         DBTableColumn col = this.createAndAppendColumn(columnName, type, size, required, defValue);
         if (enumValue!=null)
-            col.setEnumOptions(enumValue.getClass());
+            col.setEnumOptions((Class<Enum<?>>)enumValue.getClass());
         return col;
     }
     
@@ -453,7 +456,7 @@ public class DBTable extends DBRowSet implements Cloneable
             keyDataType =DataType.INTEGER;
         DBTableColumn referenceColumn = addColumn(name, keyDataType, keyCol.getSize(), required, options); 
         // Adapter foreign key
-        String fkName = getName() + "_" + name.replace("_ID", "_FK");
+        String fkName = StringUtils.concat(getName(), "_", name.replace("_ID", "_FK"));
         DBRelation relation = db.addRelation(fkName, referenceColumn.referenceOn(keyCol));
         if (cascadeAction!=null)
             relation.setOnDeleteAction(cascadeAction);
@@ -793,9 +796,82 @@ public class DBTable extends DBRowSet implements Cloneable
         }
         return Collections.unmodifiableList(relations);        
     }
+    
+    /**
+     * Determines whether a record is references by other records through a foreign-key relation or not
+     * @param key the key the record to be deleted
+     * @param context the db context
+     * @return true if the record is references or false otherwise
+     */
+    public boolean hasExistingReferences(Object[] key, DBContext context)
+    {
+        // Check all relations
+        if (getPrimaryKey()==null)
+            return false;
+        if (key==null || getPrimaryKey().getColumnCount()!=key.length)
+            throw new InvalidArgumentException("key", key);
+        // Find all relations
+        List<DBRelation> relations = db.findRelationsOn(this);
+        for (DBRelation rel : relations)
+        {   // Found a reference on RowSet
+            DBReference[] refs = rel.getReferences();
+            DBRowSet rs = refs[0].getSourceColumn().getRowSet();
+            if (getForeignRecordCount(rs, refs, key, context) > 0)
+                return true;
+        }
+        // no references
+        return false;
+    }
+    
+    /**
+     * Returns a list of all tables that contain records which reference this record
+     * @param key the key the record to be deleted
+     * @param context the db context
+     * @return this list of all tables that contain a refernce to this record
+     */
+    public List<DBTable> getExistingReferenceTables(Object[] key, DBContext context)
+    {
+        // Check all relations
+        if (getPrimaryKey()==null)
+            return null;
+        if (key==null || getPrimaryKey().getColumnCount()!=key.length)
+            throw new InvalidArgumentException("key", key);
+        // Find all relations
+        List<DBTable> tables = new ArrayList<DBTable>();
+        List<DBRelation> relations = db.findRelationsOn(this);
+        for (DBRelation rel : relations)
+        {   // Found a reference on RowSet
+            DBReference[] refs = rel.getReferences();
+            DBRowSet rs = refs[0].getSourceColumn().getRowSet();
+            if (!(rs instanceof DBTable))
+            {   log.info("ForeignKey Rowset {} is not a DBTable", rs.getName());
+                continue;
+            }
+            if (getForeignRecordCount(rs, refs, key, context) > 0)
+            {
+                tables.add((DBTable)rs);
+            }
+        }
+        // no references
+        return tables;
+    }
+    
+    private int getForeignRecordCount(DBRowSet rs, DBReference[] refs, Object[] parentKey, DBContext context)
+    {
+        // SELECT count(*) FROM rs WHERE key=parentKey
+        DBCommand cmd = db.createCommand();
+        cmd.select(rs.count());
+        for (int i=0; i<parentKey.length; i++)
+        {
+            cmd.where(refs[i].getSourceColumn().is(parentKey[i]));
+        }
+        return context.getUtils().querySingleInt(cmd);
+    }
 
     /**
      * validates a column value
+     * @param column the column
+     * @param value the value to validate
      * @return the validated (possibly converted) value
      */
     protected Object validateValue(DBTableColumn column, Object value)
@@ -826,13 +902,12 @@ public class DBTable extends DBRowSet implements Cloneable
             // check default
             DBColumn column = columns.get(i);
             // getDefaultValue
-            Object value = ((DBTableColumn)column).getRecordDefaultValue(conn);
-            if (ObjectUtils.isEmpty(value))
+            Object value = ((DBTableColumn)column).getRecordDefaultValue(record, conn);
+            if (value!=ObjectUtils.NO_VALUE && ObjectUtils.isEmpty(value))
                 continue;
             // Initial value
             // Don't set to modified: record.modifyValue(i, value, false); 
             fields[i] = value;
         }
-    }
-    
+    }    
 }
\ No newline at end of file
