@@ -24,7 +24,6 @@ import org.apache.empire.commons.Options;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.Column;
 import org.apache.empire.data.DataType;
-import org.apache.empire.db.expr.column.DBValueExpr;
 import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.ItemExistsException;
 import org.apache.empire.exceptions.NotSupportedException;
@@ -36,7 +35,7 @@ import org.w3c.dom.Element;
  * This class represents a database view.
  * It contains methods to get and update records from the database
  */
-public abstract class DBView extends DBRowSet
+public abstract class DBView extends DBRowSet implements Cloneable
 {
     // *Deprecated* private static final long serialVersionUID = 1L;
 
@@ -50,14 +49,16 @@ public abstract class DBView extends DBRowSet
       
         protected final DBColumnExpr expr;
         protected final DataType     dataType;
-        protected final DBColumn     sourceColumn;
+        protected final DBColumn     updateColumn;
         protected final double       size;
 
         /**
          * Constructs a DBViewColumn object set the specified parameters to this object.
          * 
          * @param view the DBView object
+         * @param name the column name
          * @param expr the DBColumnExpr of the source table
+         * @param size the column size
          */
         protected DBViewColumn(DBView view, String name, DBColumnExpr expr, double size)
         { // call base
@@ -70,16 +71,32 @@ public abstract class DBView extends DBRowSet
                 exprType= DataType.INTEGER;
             this.dataType = exprType;
             // Source Column
-            this.sourceColumn = expr.getSourceColumn();
+            this.updateColumn = expr.getUpdateColumn();
             // from update column
-            this.size = (sourceColumn!=null ? sourceColumn.getSize() : size);
+            this.size = (size==0.0d && updateColumn!=null ? updateColumn.getSize() : size);
             // Copy enumType
             Class<Enum<?>> enumType = expr.getEnumType();
             if (enumType!=null)
                 setAttribute(Column.COLATTR_ENUMTYPE, enumType);
-            // Add to view
-            if (view != null)
-                view.addColumn(this);
+        }
+
+        /**
+         * Copy constructor
+         * @param view the copied view
+         * @param other the column to copy
+         */
+        protected DBViewColumn(DBView view, DBViewColumn other)
+        { // call base
+            super(view, other.getName());
+            // set Expression
+            this.expr = other.expr;
+            this.dataType = other.dataType;
+            this.updateColumn = other.updateColumn;
+            this.size = other.size;
+            // Copy enumType
+            Class<Enum<?>> enumType = other.getEnumType();
+            if (enumType!=null)
+                setAttribute(Column.COLATTR_ENUMTYPE, enumType);
         }
         
         public DBColumnExpr getSourceColumnExpr()
@@ -107,19 +124,17 @@ public abstract class DBView extends DBRowSet
         @Override
         public boolean isAutoGenerated()
         {
-            if (sourceColumn==null)
+            if (updateColumn==null)
                 return false;
-            return sourceColumn.isAutoGenerated();
+            return updateColumn.isAutoGenerated();
         }
 
         @Override
         public boolean isReadOnly()
         {   
-            if (getView().isUpdateable()==false)
-                return true;
             // Check ReadOnly attribute 
-            if (sourceColumn!=null)
-                return sourceColumn.isReadOnly();
+            if (updateColumn!=null)
+                return updateColumn.isReadOnly();
             // AUTOINC's are read only
             return isAutoGenerated();
         }
@@ -127,21 +142,25 @@ public abstract class DBView extends DBRowSet
         @Override
         public boolean isRequired()
         {   
-            if (getView().isUpdateable()==false)
-                return false;
             // Check update Column
-            if (sourceColumn==null)
+            if (updateColumn==null)
                 return false;
-            return sourceColumn.isRequired();
+            return updateColumn.isRequired();
         }    
 
+        /* 
+         * Obsolete: Type is copied in constructor
+         *         
         @Override
-        public Object validateValue(Object value)
+        public Class<Enum<?>> getEnumType()
         {
-            if (sourceColumn==null)
-                return value;
-            return sourceColumn.validateValue(value);
+            Class<Enum<?>> enumType = super.getEnumType(); 
+            if (enumType!=null)
+                return enumType;  
+            // otherwise 
+            return expr.getEnumType(); 
         }
+         */
 
         @Override
         public Object getAttribute(String name)
@@ -149,9 +168,7 @@ public abstract class DBView extends DBRowSet
             if (attributes != null && attributes.contains(name))
                 return attributes.get(name);
             // Otherwise ask expression
-            if (sourceColumn==null)
-                return null;
-            return sourceColumn.getAttribute(name);
+            return expr.getAttribute(name);
         }
 
         @Override
@@ -160,9 +177,15 @@ public abstract class DBView extends DBRowSet
             if (options != null)
                 return options;
             // Otherwise ask expression
-            if (sourceColumn==null)
-                return null;
-            return sourceColumn.getOptions();
+            return expr.getOptions();
+        }
+
+        @Override
+        public Object validateValue(Object value)
+        {
+            if (updateColumn==null)
+                return value;
+            return updateColumn.validateValue(value);
         }
 
         @Override
@@ -171,8 +194,8 @@ public abstract class DBView extends DBRowSet
             Element elem = XMLUtil.addElement(parent, "column");
             elem.setAttribute("name", name);
             // set default attributes
-            if (sourceColumn != null)
-                elem.setAttribute("source", sourceColumn.getFullName());
+            if (updateColumn != null)
+                elem.setAttribute("source", updateColumn.getFullName());
             // size
             double size = getSize();
             if (size > 0)
@@ -187,17 +210,61 @@ public abstract class DBView extends DBRowSet
             // done
             return elem;
         }
-
+        
+        /**
+         * sets the options from an enum class
+         * @param enumType the enum type
+         */
+        public void setEnumOptions(Class<? extends Enum<?>> enumType)
+        {
+            // Enum special treatment
+            log.debug("Adding enum options of type {} for column {}.", enumType.getName(), getName());            
+            this.options = new Options(enumType);
+            // set enumType
+            setAttribute(Column.COLATTR_ENUMTYPE, enumType);
+        }
     }
 
     private static AtomicInteger viewCount = new AtomicInteger(0);
+    /**
+     * Automatically generates a new alias for this Object 
+     * @param prefix the alias prefix
+     * @return an alias consisting of the prefix and a unique number
+     */
+    protected String generateAlias(String prefix)
+    {
+        return prefix + String.valueOf(viewCount.incrementAndGet());
+    }
 
-    private String               name;
+    private final String         name;
     private String               alias;
     private DBViewColumn[]       keyColumns;
     private boolean              updateable;                      // true if the view is updateable
     private Boolean              quoteName = null;
-
+    
+    /**
+     * Creates a view object for a given view in the database.
+     * 
+     * @param name the name of the view
+     * @param db the database this view belongs to.
+     * @param isUpdateable true if the records of this view can be updated 
+     * @param alias the view alias
+     */
+    public DBView(String name, DBDatabase db, boolean isUpdateable, String alias)
+    { // Set the column expressions
+        super(db);
+        // generate alias
+        if (StringUtils.isEmpty(alias))
+            alias = generateAlias("v");
+        // Set Name and Alias
+        this.name = name;
+        this.alias = alias;
+        this.updateable = isUpdateable;
+        // Add View to Database
+        if (db != null && name != null)
+            db.addView(this);
+    }
+    
     /**
      * Creates a view object for a given view in the database.
      * 
@@ -207,14 +274,7 @@ public abstract class DBView extends DBRowSet
      */
     public DBView(String name, DBDatabase db, boolean isUpdateable)
     { // Set the column expressions
-        super(db);
-        // Set Name and Alias
-        this.name = name;
-        this.alias = "v" + String.valueOf(viewCount.incrementAndGet());
-        this.updateable = isUpdateable;
-        // Add View to Database
-        if (db != null && name != null)
-            db.addView(this);
+        this(name, db, isUpdateable, null);
     }
 
     /**
@@ -228,6 +288,74 @@ public abstract class DBView extends DBRowSet
         this(name, db, false);
     }
 
+    /**
+     * Allows a second instance of a view with a different alias
+     * @return another instance of the view
+     */
+    @Override
+    public Object clone() throws CloneNotSupportedException 
+    {
+        DBView clone = (DBView) super.clone();
+        initClonedFields(clone);
+        // set key columns
+        clone.keyColumns = cloneKeyColumns(clone);
+        // set new alias
+        clone.alias = generateAlias("v");
+        // done
+        log.info("clone: View {} cloned! Alias old={} new={}", name, alias, clone.alias);
+        db.addView(clone);
+        return clone;
+    }
+    
+    /**
+     * Allows a second instance of a view with a different alias
+     * @param newAlias the new alias
+     * @return another instance of the view
+     */
+    @SuppressWarnings("unchecked")
+    public <T extends DBView> T clone(String newAlias) 
+    {
+        try {
+            DBView clone = (DBView) super.clone();
+            initClonedFields(clone);
+            // set key columns
+            clone.keyColumns = cloneKeyColumns(clone);
+            // set new alias
+            if (StringUtils.isEmpty(newAlias))
+                clone.alias = generateAlias("v");
+            else
+                clone.alias = newAlias;
+            // done
+            log.info("clone: View {} cloned! Alias old={} new={}", name, alias, clone.alias);
+            db.addView(clone);
+            return (T)clone;
+        } catch (CloneNotSupportedException e) {
+            // unable to clone table
+            log.error("Unable to clone table {}", getName());
+            throw new RuntimeException(e);
+        }
+    }
+    
+    @Override
+    protected DBColumn cloneColumn(DBRowSet clone, DBColumn sourceColumn)
+    {
+        DBViewColumn newCol = new DBViewColumn((DBView)clone, (DBViewColumn)sourceColumn);
+        addViewColumn(newCol);
+        return newCol;
+    }
+
+    protected DBViewColumn[] cloneKeyColumns(DBView clone)
+    {
+        if (this.keyColumns==null)
+            return null;
+        // clone columns
+        DBViewColumn[] columns = new DBViewColumn[keyColumns.length];
+        for (int i=0; i<keyColumns.length; i++)
+            columns[i] = (DBViewColumn)clone.getColumn(getColumnIndex(keyColumns[i]));
+        // Create clone
+        return columns;
+    }
+
     /**
      * Returns an array of all key columns.
      * @return an array of all key columns
@@ -321,7 +449,7 @@ public abstract class DBView extends DBRowSet
      * 
      * @param col a view column object
      */
-    protected void addColumn(DBViewColumn col)
+    protected void addViewColumn(DBViewColumn col)
     { // find column by name
         if (col == null || col.getRowSet() != this)
             throw new InvalidArgumentException("col", col);
@@ -334,26 +462,17 @@ public abstract class DBView extends DBRowSet
     /**
      * Adds a column to the view.
      * 
-     * @param columnName name of the column in the view
-     * @param dataType the data type of the column
-     * @param size the size of the column
-     * @return true if the column was successfully added or false otherwise
+     * @param columnName the column name
+     * @param sourceExpression the source column expression
+     * @param size the size
+     * @return the new column
      */
-    protected final DBViewColumn addColumn(String columnName, DataType dataType, double size)
-    {   // find column by name
-        return new DBViewColumn(this, columnName, new DBValueExpr(db, null, dataType), size);
-    }
-
-    /**
-     * Adds a column to the view.
-     * 
-     * @param columnName name of the column in the view
-     * @param dataType the data type of the column
-     * @return true if the column was successfully added or false otherwise
-     */
-    protected final DBViewColumn addColumn(String columnName, DataType dataType)
-    {   // find column by name
-        return new DBViewColumn(this, columnName, new DBValueExpr(db, null, dataType), 0.0d);
+    protected DBViewColumn createAndAppendColumn(String columnName, DBColumnExpr sourceExpression, double size)
+    {
+        DBViewColumn viewColumn = new DBViewColumn(this, columnName, sourceExpression, size);
+        // add now
+        addViewColumn(viewColumn);
+        return viewColumn;
     }
 
     /**
@@ -361,26 +480,62 @@ public abstract class DBView extends DBRowSet
      * 
      * @param columnName name of the column in the view
      * @param columnExpr the column expression that builds the column
-     * @return true if the column was successfully added or false otherwise
+     * @return the new column
      */
     protected final DBViewColumn addColumn(String columnName, DBColumnExpr columnExpr)
     {   // find column by name
-        return new DBViewColumn(this, columnName, columnExpr, 0.0d);
+        return createAndAppendColumn(columnName, columnExpr, 0.0d);
+    }
+    
+    /**
+     * Adds a column to the view.
+     * 
+     * @param columnName name of the column in the view
+     * @param dataType the data type of the column
+     * @param size the size of the column
+     * @return the new column
+     */
+    protected final DBViewColumn addColumn(String columnName, DataType dataType, double size)
+    {   // find column by name
+        return createAndAppendColumn(columnName, db.getValueExpr(null, dataType), size);
     }
 
     /**
-     * Adds a column to the view based on an existing column in another table or view.
+     * Adds a column to the view.
      * 
-     * @param sourceColumn existing column in another table or view  
-     * @return the view column object
+     * @param columnName name of the column in the view
+     * @param dataType the data type of the column
+     * @return the new column
+     */
+    protected final DBViewColumn addColumn(String columnName, DataType dataType)
+    {   // find column by name
+        return createAndAppendColumn(columnName, db.getValueExpr(null, dataType), 0.0d);
+    }
+
+    /**
+     * Adds a column to the view based on an existing column in another table.
+     * 
+     * @param sourceColumn existing column in another table  
+     * @return the new column
      */
     protected final DBViewColumn addColumn(DBTableColumn sourceColumn)
     {   // find column by name
-        return new DBViewColumn(this, sourceColumn.getName(), sourceColumn, 0.0d);
+        return createAndAppendColumn(sourceColumn.getName(), sourceColumn, sourceColumn.getSize());
     }
 
     /**
-     * This function searchs for equal columns given by the specified DBColumnExpr object.
+     * Adds a column to the view based on an existing column in another view.
+     * 
+     * @param sourceColumn existing column in another view  
+     * @return the new column
+     */
+    protected final DBViewColumn addColumn(DBViewColumn sourceColumn)
+    {   // find column by name
+        return createAndAppendColumn(sourceColumn.getName(), sourceColumn, sourceColumn.getSize());
+    }
+
+    /**
+     * This function searches for equal columns given by the specified DBColumnExpr object.
      * 
      * @param expr the DBColumnExpr object
      * @return the located column (only DBViewColumn objects)
