@@ -92,8 +92,8 @@ public class DBModelParser
     
     /**
      * Creates a new Model Checker
-     * @param catalog
-     * @param schema
+     * @param catalog the catalog
+     * @param schema the schema
      */
     public DBModelParser(String catalog, String schema)
     {
@@ -202,7 +202,8 @@ public class DBModelParser
      * collects table and view information from database meta data
      * @param dbMeta the database meta data
      * @param tablePattern the table pattern
-     * @throws SQLException
+     * @throws SQLException thrown if a database access error occurs
+     * @return the table and view count
      */
     protected int collectTablesAndViews(DatabaseMetaData dbMeta, String tablePattern)
         throws SQLException
@@ -218,7 +219,7 @@ public class DBModelParser
                 String tableType = dbTables.getString("TABLE_TYPE");
                 if (isSystemTable(tableName, dbTables))
                 {   // ignore system table
-                    DBModelParser.log.info("Ignoring system table " + tableName);
+                    log.info("Ignoring system table " + tableName);
                     continue;
                 }
                 if ("VIEW".equalsIgnoreCase(tableType))
@@ -235,6 +236,9 @@ public class DBModelParser
 
     /**
      * collects column information from database meta data for each table
+     * @param dbMeta the database meta data
+     * @throws SQLException thrown if a database access error occurs
+     * @return the column count
      */
     protected int collectColumns(DatabaseMetaData dbMeta)
             throws SQLException
@@ -258,6 +262,10 @@ public class DBModelParser
 
     /**
      * collects column information from database meta data for whole schema
+     * @param dbMeta the database meta data
+     * @param tablePattern the table pattern
+     * @throws SQLException thrown if a database access error occurs
+     * @return the column count
      */
     protected int collectColumns(DatabaseMetaData dbMeta, String tablePattern)
         throws SQLException
@@ -270,7 +278,7 @@ public class DBModelParser
                 String tableName = dbColumns.getString("TABLE_NAME");
                 DBRowSet t = getTable(tableName);
                 if (t == null)
-                {   log.error("Table not found: {}", tableName);
+                {   log.info("Table \"{}\" for column \"{}\" is not available!", tableName, dbColumns.getString("COLUMN_NAME"));
                     continue;
                 }
                 addColumn(t, dbColumns);
@@ -285,7 +293,8 @@ public class DBModelParser
     /**
      * collects primary key information from database meta data
      * @param dbMeta the database meta data
-     * @throws SQLException
+     * @throws SQLException thrown if a database access error occurs
+     * @return the primary key count
      */
     protected int collectPrimaryKeys(DatabaseMetaData dbMeta)
         throws SQLException
@@ -340,7 +349,8 @@ public class DBModelParser
     /**
      * collects foreign key information from database meta data
      * @param dbMeta the database meta data
-     * @throws SQLException
+     * @throws SQLException thrown if a database access error occurs
+     * @return the foreign key count
      */
     protected int collectForeignKeys(DatabaseMetaData dbMeta)
             throws SQLException
@@ -358,7 +368,8 @@ public class DBModelParser
      * collects foreign key information from database meta data
      * @param dbMeta the database meta data
      * @param tablePattern the table pattern
-     * @throws SQLException
+     * @throws SQLException thrown if a database access error occurs
+     * @return the foreign key count
      */
     protected int collectForeignKeys(DatabaseMetaData dbMeta, String tablePattern)
         throws SQLException
@@ -479,7 +490,7 @@ public class DBModelParser
         }
         else
         {   // Unknown type
-            log.error("Unknown Object Type {}", t.getClass().getName());
+            log.warn("Unknown Object Type {}", t.getClass().getName());
             col = null;
         }
         // done
@@ -502,12 +513,12 @@ public class DBModelParser
                 catch (Exception e)
                 {
                     String name = rs.getString("COLUMN_NAME");
-                    DBModelParser.log.error("Failed to parse decimal digits for column " + name);
+                    log.error("Failed to parse decimal digits for column " + name);
                 }
             }
             // make integer?
             if (colSize < 1.0d)
-            { // Turn into an integer
+            {   // Turn into an integer
                 empireType = DataType.INTEGER;
             }
         } 
@@ -606,9 +617,9 @@ public class DBModelParser
                 break;
             default:
                 empireType = DataType.UNKNOWN;
-                DBModelParser.log.warn("SQL column type " + sqlType + " not supported.");
+                log.warn("SQL column type " + sqlType + " not supported.");
         }
-        DBModelParser.log.debug("Mapping date type " + String.valueOf(sqlType) + " to " + empireType);
+        log.debug("Mapping date type " + String.valueOf(sqlType) + " to " + empireType);
         return empireType;
     }
 }
