@@ -44,6 +44,7 @@ import org.apache.empire.db.DBCommand;
 import org.apache.empire.db.DBCommandExpr;
 import org.apache.empire.db.DBDDLGenerator.DDLActionType;
 import org.apache.empire.db.DBDatabase;
+import org.apache.empire.db.DBObject;
 import org.apache.empire.db.DBRelation;
 import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.DBSQLBuilder;
@@ -51,7 +52,6 @@ import org.apache.empire.db.DBSQLScript;
 import org.apache.empire.db.DBTable;
 import org.apache.empire.db.DBTableColumn;
 import org.apache.empire.db.exceptions.EmpireSQLException;
-import org.apache.empire.db.exceptions.QueryFailedException;
 import org.apache.empire.db.validation.DBModelChecker;
 import org.apache.empire.db.validation.DBModelParser;
 import org.apache.empire.exceptions.InvalidArgumentException;
@@ -73,10 +73,11 @@ public abstract class DBMSHandlerBase implements DBMSHandler
                                                                     '/', '.', '-', ',', '+', '*', ')', '(',
                                                                     '\'', '&', '%', '!', ' '
                                                                   };        
-    protected static final String[] GENERAL_SQL_KEYWORDS = new String[] { "user", "group", 
+    protected static final String[] GENERAL_SQL_KEYWORDS = new String[] {
                                                            "table", "column", "view", "index", "constraint", 
                                                            "select", "udpate", "insert", "alter", "delete", 
-                                                           "order" };        
+                                                           "join", "on", "group", "by", "order", "asc", "desc", "all", 
+                                                           "with", "user" };        
     protected final Set<String> reservedSQLKeywords;
 
     // Postfix for auto-generated Sequence names
@@ -229,13 +230,35 @@ public abstract class DBMSHandlerBase implements DBMSHandler
     /**
      * Constructor
      */
-    protected DBMSHandlerBase()
+    protected DBMSHandlerBase(String[] specificSqlKeywords)
     {
         // Initialize List of reserved Keywords
-        reservedSQLKeywords = new HashSet<String>(GENERAL_SQL_KEYWORDS.length);
-        for (String keyWord:GENERAL_SQL_KEYWORDS){
-             reservedSQLKeywords.add(keyWord);
+        int capacity = GENERAL_SQL_KEYWORDS.length + specificSqlKeywords.length;
+        reservedSQLKeywords = new HashSet<String>(capacity);
+        for (String keyWord : GENERAL_SQL_KEYWORDS) {
+            reservedSQLKeywords.add(keyWord);
         }
+        // Initialize List of reserved Keywords
+        for (String keyWord : specificSqlKeywords) {
+            addSQLKeyword(keyWord);
+        }
+    }
+    
+    /**
+     * Constructor
+     */
+    protected DBMSHandlerBase()
+    {
+        this(new String[0]);
+    }
+
+    /**
+     * Adds an additional SQL Keyword to the keyword list
+     * @param keyWord
+     */
+    protected void addSQLKeyword(String keyWord)
+    {
+        reservedSQLKeywords.add(keyWord.toLowerCase());
     }
 
     /**
@@ -272,7 +295,7 @@ public abstract class DBMSHandlerBase implements DBMSHandler
         try {
             querySingleValue(sql.toString(), null, DataType.INTEGER, conn);
             return true;
-        } catch(QueryFailedException e) {
+        } catch(SQLException e) {
             // Database does not exist
             return false;
         }
@@ -347,7 +370,8 @@ public abstract class DBMSHandlerBase implements DBMSHandler
      * @param name the name which to check
      * @return true if the name needs to be quoted or false otherwise
      */
-    public boolean detectQuoteName(String name)
+    @Override
+    public boolean detectQuoteName(DBObject object, String name)
     {
         // Check for reserved names
         if (reservedSQLKeywords.contains(name.toLowerCase()))
@@ -379,7 +403,7 @@ public abstract class DBMSHandlerBase implements DBMSHandler
     public void appendObjectName(DBSQLBuilder sql, String name, Boolean useQuotes)
     {
         if (useQuotes==null)
-            useQuotes = detectQuoteName(name);
+            useQuotes = detectQuoteName(null, name);
         // Check whether to use quotes or not
         if (useQuotes)
             sql.append(getSQLPhrase(DBSqlPhrase.SQL_QUOTES_OPEN));
@@ -485,7 +509,24 @@ public abstract class DBMSHandlerBase implements DBMSHandler
             return ts;
         }
         // Other types
-        throw new NotSupportedException(this, "getColumnAutoValue() for "+type);
+        throw new NotSupportedException(this, "getColumnAutoValue for "+column.getFullName());
+    }
+
+    /**
+     * Returns an expression that ignores the case of a column expression
+     * Only for text columns.
+     * Default is upper(expr)
+     * @param expr the expression for which to ignore the case
+     * @return the ignore case expression or the expression itself
+     */
+    @Override
+    public DBColumnExpr getIgnoreCaseExpr(DBColumnExpr expr)
+    {
+        if (expr==null || !expr.getDataType().isText())
+        {   // not a CHAR or VARCHAR
+            return expr; 
+        }
+        return expr.upper();
     }
     
     /**
@@ -506,24 +547,33 @@ public abstract class DBMSHandlerBase implements DBMSHandler
     public Object getResultValue(ResultSet rset, int columnIndex, DataType dataType)
         throws SQLException
     {
+        // Special handing of DATE, TIME, DATETIME and TIMESTAMP
+        if (dataType == DataType.DATE)
+        {   // use getDate() (do not use getObject()!)
+            return rset.getDate(columnIndex);
+        }
+        if (dataType == DataType.TIME)
+        {   // use getTime() (do not use getObject()!)
+            return rset.getTime(columnIndex);
+        }
         if (dataType == DataType.DATETIME || dataType == DataType.TIMESTAMP)
-        { // Get Timestamp (do not use getObject()!) 
+        {   // use getTimestamp() (do not use getObject()!) 
             return rset.getTimestamp(columnIndex);
         } 
-        else if (dataType == DataType.CLOB)
-        {
+        // Check for character large object
+        if (dataType == DataType.CLOB)
+        {   // Get string from character large object
             java.sql.Clob clob = rset.getClob(columnIndex);
             return ((clob != null) ? clob.getSubString(1, (int) clob.length()) : null);
         } 
-        else if (dataType == DataType.BLOB)
-        { // Get bytes of a binary large object
+        // Check for binary large object
+        if (dataType == DataType.BLOB)
+        {   // Get bytes of a binary large object
             java.sql.Blob blob = rset.getBlob(columnIndex);
             return ((blob != null) ? blob.getBytes(1, (int) blob.length()) : null);
-        } 
-        else
-        {
-            return rset.getObject(columnIndex);
         }
+        // default
+        return rset.getObject(columnIndex);
     }
     
     /**
@@ -707,6 +757,7 @@ public abstract class DBMSHandlerBase implements DBMSHandler
      */
     @Override
     public Object querySingleValue(String sqlCmd, Object[] sqlParams, DataType dataType, Connection conn)
+        throws SQLException
     {
         ResultSet rs = null;
         try
@@ -722,9 +773,6 @@ public abstract class DBMSHandlerBase implements DBMSHandler
             }
             // Read value
             return getResultValue(rs, 1, dataType);
-        } catch (SQLException sqle) 
-        {   // Error
-            throw new QueryFailedException(this, sqlCmd, sqle);
         } finally {
             // Cleanup
             closeResultSet(rs);
