@@ -42,17 +42,20 @@ public enum DBSqlPhrase
     SQL_BOOLEAN_TRUE        ("1"),          // Oracle Y
     SQL_BOOLEAN_FALSE       ("0"),          // Oracle N
     SQL_CURRENT_DATE        ("CURRENT_DATE"), 
-    SQL_DATE_PATTERN        ("yyyy-MM-dd"),   // SimpleDateFormat
     SQL_DATE_TEMPLATE       ("TO_DATE('{0}', 'YYYY-MM-DD')"),  // MSSql: convert(date, '{0}', 111)
     SQL_CURRENT_TIME        ("CURRENT_TIME"), // MSSql: CONVERT(time, getdate());
-    SQL_TIME_PATTERN        ("HH:mm:ss"),     // SimpleDateFormat
     SQL_TIME_TEMPLATE       ("'{0}'"),        // MSSql: convert(time, '{0}')
-    SQL_DATETIME_PATTERN    ("yyyy-MM-dd HH:mm:ss.SSS"), // SimpleDateFormat        
+    SQL_CURRENT_DATETIME    ("CURRENT_TIMESTAMP"),
     SQL_DATETIME_TEMPLATE   ("TO_DATE('{0}', 'YYYY-MM-DD HH24:MI:SS')"), // Oracle
-    SQL_CURRENT_TIMESTAMP   ("systimestamp"),                   // Oracle
-    SQL_TIMESTAMP_PATTERN   ("yyyy-MM-dd HH:mm:ss.SSS"),        // SimpleDateFormat
+    SQL_CURRENT_TIMESTAMP   ("CURRENT_TIMESTAMP"),
     SQL_TIMESTAMP_TEMPLATE  ("TO_TIMESTAMP('{0}', 'YYYY.MM.DD HH24:MI:SS.FF')"), // Oracle
 
+    // DateTimeFormatter patterns
+    SQL_DATE_PATTERN        ("yyyy-MM-dd"),   
+    SQL_TIME_PATTERN        ("HH:mm:ss"),
+    SQL_DATETIME_PATTERN    ("yyyy-MM-dd HH:mm:ss"),        
+    SQL_TIMESTAMP_PATTERN   ("yyyy-MM-dd HH:mm:ss.SSS"), // or "yyyy-MM-dd HH:mm:ss.SSS " in order to limit to 3 fractions
+    
     // functions
     SQL_FUNC_COALESCE       ("coalesce(?, {0})"),       // Oracle: nvl(?, {0})
     SQL_FUNC_SUBSTRING      ("substring(?, {0:INTEGER})"),
@@ -69,6 +72,7 @@ public enum DBSqlPhrase
     SQL_FUNC_RTRIM          ("rtrim(?)"),
     SQL_FUNC_ESCAPE         ("? escape {0:VARCHAR}"),
     SQL_FUNC_CONCAT         ("concat(?)"),
+    SQL_FUNC_CONTAINS       ("contains(?, {0})"),
     
     // Numeric
     SQL_FUNC_ABS            ("abs(?)"),
