@@ -39,6 +39,7 @@ import org.apache.empire.db.DBSQLScript;
 import org.apache.empire.db.DBTable;
 import org.apache.empire.db.DBTableColumn;
 import org.apache.empire.db.exceptions.EmpireSQLException;
+import org.apache.empire.db.exceptions.QueryFailedException;
 import org.apache.empire.db.validation.DBModelChecker;
 import org.apache.empire.db.validation.DBModelParser;
 import org.apache.empire.dbms.DBMSFeature;
@@ -57,6 +58,9 @@ public class DBMSHandlerMSSQL extends DBMSHandlerBase
 {
     // *Deprecated* private static final long serialVersionUID = 1L;
     private static final Logger log = LoggerFactory.getLogger(DBMSHandlerMSSQL.class);
+
+    // Additional SQL-Server Keywords
+    protected static final String[] MSSQL_SQL_KEYWORDS = new String[] { "case", "when", "type", "key", "plan" };        
   
     /**
      * Provides a DBCommand implementation for Microsoft SQL-Server
@@ -172,18 +176,14 @@ public class DBMSHandlerMSSQL extends DBMSHandlerBase
     private boolean useDateTime2 = true;
     
     protected DBDDLGenerator<?> ddlGenerator = null; // lazy creation
-
-    protected static final String[] MSSQL_SQL_KEYWORDS = new String[] { "type", "key", "plan" };        
     
     /**
      * Constructor for the MSSQL database dbms.<br>
      */
     public DBMSHandlerMSSQL()
     {
-        // Initialize List of reserved Keywords
-        for (String keyWord:MSSQL_SQL_KEYWORDS){
-             reservedSQLKeywords.add(keyWord);
-        }
+        // Add additional Keywords
+        super(MSSQL_SQL_KEYWORDS);
     }
 
     public String getDatabaseName()
@@ -395,17 +395,15 @@ public class DBMSHandlerMSSQL extends DBMSHandlerBase
             // data types
             case SQL_BOOLEAN_TRUE:            return "1";
             case SQL_BOOLEAN_FALSE:           return "0";
-            case SQL_CURRENT_DATE:            return "convert(char, getdate(), 111)";
-            case SQL_DATE_PATTERN:            return "yyyy-MM-dd";
+            case SQL_CURRENT_DATE:            return "convert(date, getdate())";
             case SQL_DATE_TEMPLATE:           return "convert(date, '{0}', 111)";
-            case SQL_CURRENT_TIME:            return "convert(time, getdate())";
-            case SQL_TIME_PATTERN:            return "HH:mm:ss";
-            case SQL_TIME_TEMPLATE:           return "convert(time, '{0}')";
-            case SQL_DATETIME_PATTERN:        return "yyyy-MM-dd HH:mm:ss.SSS";
+            case SQL_CURRENT_TIME:            return "convert(time, convert(varchar, getdate(), 120), 111)";
+            case SQL_TIME_TEMPLATE:           return "convert(time, '{0}', 108)";
+            case SQL_CURRENT_DATETIME:        return isUseDateTime2() ? "convert(datetime2, convert(varchar, getdate(), 120), 111)"
+                                                                      : "convert(datetime,  convert(varchar, getdate(), 120), 111)";
             case SQL_DATETIME_TEMPLATE:       return isUseDateTime2() ? "convert(datetime2, '{0}', 121)"
                                                                       : "convert(datetime,  '{0}', 121)";
             case SQL_CURRENT_TIMESTAMP:       return "getdate()";
-            case SQL_TIMESTAMP_PATTERN:       return "yyyy-MM-dd HH:mm:ss.SSS";
             case SQL_TIMESTAMP_TEMPLATE:      return isUseDateTime2() ? "convert(datetime2, '{0}', 121)"
                                                                       : "convert(datetime,  '{0}', 121)";
             // functions
@@ -552,8 +550,13 @@ public class DBMSHandlerMSSQL extends DBMSHandlerBase
     {
         // Supports sequences?
         if (column.getDataType()==DataType.UNIQUEID)
-        {
-            return querySingleValue("select newid()", null, DataType.UNIQUEID, conn);
+        {   // create new unique-id
+            String sqlCmd = "select newid()";
+            try {
+                return querySingleValue(sqlCmd, null, DataType.UNIQUEID, conn);
+            } catch (SQLException sqle) {
+                throw new QueryFailedException(this, sqlCmd, null, sqle);
+            } 
         }
         return super.getColumnAutoValue(db, column, conn);
     }
